@@ -36,7 +36,15 @@ if [ "$VOLUME_MODE" == "block" ]; then
     /usr/bin/cdi-cloner -v=3 -alsologtostderr -content-type blockdevice-clone -upload-bytes $UPLOAD_BYTES -mount $MOUNT_POINT
 else
     pushd $MOUNT_POINT
-    UPLOAD_BYTES=$(du -sb . | cut -f1)
+    if [ "$PREALLOCATION" == "true" ]; then
+        echo "Preallocating filesystem, uploading all bytes"
+        UPLOAD_BYTES=$(du -sb . | cut -f1)
+    else
+        # If we don't preallocate, we need to make sure we don't upload more than
+        # the filesystem actually contains.
+        echo "Not preallocating filesystem, uploading only used bytes"
+        UPLOAD_BYTES=$(du -s --block-size=1 . | cut -f1)
+    fi
     echo "UPLOAD_BYTES=$UPLOAD_BYTES"
 
     /usr/bin/cdi-cloner -v=3 -alsologtostderr -content-type filesystem-clone -upload-bytes $UPLOAD_BYTES -mount $MOUNT_POINT
