@@ -41,6 +41,7 @@ import (
 	"k8s.io/klog/v2"
 	aggregatorclient "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset"
 	k8sspec "k8s.io/kube-openapi/pkg/validation/spec"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	snapclient "github.com/kubernetes-csi/external-snapshotter/client/v6/clientset/versioned"
 	cdiuploadv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/upload/v1beta1"
@@ -53,6 +54,7 @@ import (
 	"kubevirt.io/containerized-data-importer/pkg/util"
 	"kubevirt.io/containerized-data-importer/pkg/util/openapi"
 	cryptowatch "kubevirt.io/containerized-data-importer/pkg/util/tls-crypto-watch"
+	cdiversion "kubevirt.io/containerized-data-importer/pkg/version"
 )
 
 const (
@@ -65,6 +67,8 @@ const (
 
 	dvMutatePath = "/datavolume-mutate"
 
+	pvcMutatePath = "/pvc-mutate"
+
 	cdiValidatePath = "/cdi-validate"
 
 	objectTransferValidatePath = "/objecttransfer-validate"
@@ -92,10 +96,11 @@ type cdiAPIApp struct {
 	bindAddress string
 	bindPort    uint
 
-	client           kubernetes.Interface
-	aggregatorClient aggregatorclient.Interface
-	cdiClient        cdiclient.Interface
-	snapClient       snapclient.Interface
+	client                  kubernetes.Interface
+	aggregatorClient        aggregatorclient.Interface
+	cdiClient               cdiclient.Interface
+	snapClient              snapclient.Interface
+	controllerRuntimeClient client.Client
 
 	privateSigningKey *rsa.PrivateKey
 
@@ -126,6 +131,7 @@ func NewCdiAPIServer(bindAddress string,
 	aggregatorClient aggregatorclient.Interface,
 	cdiClient cdiclient.Interface,
 	snapClient snapclient.Interface,
+	controllerRuntimeClient client.Client,
 	authorizor CdiAPIAuthorizer,
 	authConfigWatcher AuthConfigWatcher,
 	cdiConfigTLSWatcher cryptowatch.CdiConfigTLSWatcher,
@@ -133,17 +139,18 @@ func NewCdiAPIServer(bindAddress string,
 	installerLabels map[string]string) (CdiAPIServer, error) {
 	var err error
 	app := &cdiAPIApp{
-		bindAddress:         bindAddress,
-		bindPort:            bindPort,
-		client:              client,
-		aggregatorClient:    aggregatorClient,
-		cdiClient:           cdiClient,
-		snapClient:          snapClient,
-		authorizer:          authorizor,
-		authConfigWatcher:   authConfigWatcher,
-		cdiConfigTLSWatcher: cdiConfigTLSWatcher,
-		certWarcher:         certWatcher,
-		installerLabels:     installerLabels,
+		bindAddress:             bindAddress,
+		bindPort:                bindPort,
+		client:                  client,
+		aggregatorClient:        aggregatorClient,
+		cdiClient:               cdiClient,
+		snapClient:              snapClient,
+		controllerRuntimeClient: controllerRuntimeClient,
+		authorizer:              authorizor,
+		authConfigWatcher:       authConfigWatcher,
+		cdiConfigTLSWatcher:     cdiConfigTLSWatcher,
+		certWarcher:             certWatcher,
+		installerLabels:         installerLabels,
 	}
 
 	err = app.getKeysAndCerts()
@@ -184,6 +191,11 @@ func NewCdiAPIServer(bindAddress string,
 		return nil, errors.Errorf("failed to create DataVolume mutating webhook: %s", err)
 	}
 
+	err = app.createPvcMutatingWebhook()
+	if err != nil {
+		return nil, errors.Errorf("failed to create PVC mutating webhook: %s", err)
+	}
+
 	err = app.createCDIValidatingWebhook()
 	if err != nil {
 		return nil, errors.Errorf("failed to create CDI validating webhook: %s", err)
@@ -494,6 +506,7 @@ func (app *cdiAPIApp) composeUploadTokenAPI() {
 				if err != nil {
 					panic(fmt.Errorf("failed to build swagger: %s", err))
 				}
+				openapispec.Info.Version = cdiversion.Get().String()
 			})
 			writeJSONResponse(response, openapispec)
 		}))
@@ -512,7 +525,7 @@ func (app *cdiAPIApp) healthzHandler(req *restful.Request, resp *restful.Respons
 }
 
 func (app *cdiAPIApp) createDataVolumeValidatingWebhook() error {
-	app.container.ServeMux.Handle(dvValidatePath, webhooks.NewDataVolumeValidatingWebhook(app.client, app.cdiClient, app.snapClient))
+	app.container.ServeMux.Handle(dvValidatePath, webhooks.NewDataVolumeValidatingWebhook(app.client, app.cdiClient, app.snapClient, app.controllerRuntimeClient))
 	return nil
 }
 
@@ -521,6 +534,11 @@ func (app *cdiAPIApp) createDataVolumeMutatingWebhook() error {
 	return nil
 }
 
+func (app *cdiAPIApp) createPvcMutatingWebhook() error {
+	app.container.ServeMux.Handle(pvcMutatePath, webhooks.NewPvcMutatingWebhook(app.controllerRuntimeClient))
+	return nil
+}
+
 func (app *cdiAPIApp) createCDIValidatingWebhook() error {
 	app.container.ServeMux.Handle(cdiValidatePath, webhooks.NewCDIValidatingWebhook(app.cdiClient))
 	return nil
