@@ -51,7 +51,7 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/record"
 	"k8s.io/klog/v2"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	cdiv1utils "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1/utils"
@@ -192,7 +192,7 @@ const (
 	// AnnCloneType is the comuuted/requested clone type
 	AnnCloneType = AnnAPIGroup + "/cloneType"
 	// AnnCloneSourcePod name of the source clone pod
-	AnnCloneSourcePod = "cdi.kubevirt.io/storage.sourceClonePodName"
+	AnnCloneSourcePod = AnnAPIGroup + "/storage.sourceClonePodName"
 
 	// AnnUploadRequest marks that a PVC should be made available for upload
 	AnnUploadRequest = AnnAPIGroup + "/storage.upload.target"
@@ -229,10 +229,14 @@ const (
 	AnnPodNetwork = "k8s.v1.cni.cncf.io/networks"
 	// AnnPodMultusDefaultNetwork is used for specifying default Pod Network
 	AnnPodMultusDefaultNetwork = "v1.multus-cni.io/default-network"
-	// AnnPodSidecarInjection is used for enabling/disabling Pod istio/AspenMesh sidecar injection
-	AnnPodSidecarInjection = "sidecar.istio.io/inject"
-	// AnnPodSidecarInjectionDefault is the default value passed for AnnPodSidecarInjection
-	AnnPodSidecarInjectionDefault = "false"
+	// AnnPodSidecarInjectionIstio is used for enabling/disabling Pod istio/AspenMesh sidecar injection
+	AnnPodSidecarInjectionIstio = "sidecar.istio.io/inject"
+	// AnnPodSidecarInjectionIstioDefault is the default value passed for AnnPodSidecarInjection
+	AnnPodSidecarInjectionIstioDefault = "false"
+	// AnnPodSidecarInjectionLinkerd is used to enable/disable linkerd sidecar injection
+	AnnPodSidecarInjectionLinkerd = "linkerd.io/inject"
+	// AnnPodSidecarInjectionLinkerdDefault is the default value passed for AnnPodSidecarInjectionLinkerd
+	AnnPodSidecarInjectionLinkerdDefault = "disabled"
 
 	// AnnImmediateBinding provides a const to indicate whether immediate binding should be performed on the PV (overrides global config)
 	AnnImmediateBinding = AnnAPIGroup + "/storage.bind.immediate.requested"
@@ -241,8 +245,11 @@ const (
 	// be dynamically provisioned. Its value is the name of the selected node.
 	AnnSelectedNode = "volume.kubernetes.io/selected-node"
 
+	// AnnGarbageCollected is a PVC annotation indicating that the PVC was garbage collected
+	AnnGarbageCollected = AnnAPIGroup + "/garbageCollected"
+
 	// CloneUniqueID is used as a special label to be used when we search for the pod
-	CloneUniqueID = "cdi.kubevirt.io/storage.clone.cloneUniqeId"
+	CloneUniqueID = AnnAPIGroup + "/storage.clone.cloneUniqeId"
 
 	// CloneSourceInUse is reason for event created when clone source pvc is in use
 	CloneSourceInUse = "CloneSourceInUse"
@@ -313,9 +320,18 @@ const (
 	ProgressDone = "100.0%"
 
 	// AnnEventSourceKind is the source kind that should be related to events
-	AnnEventSourceKind = "cdi.kubevirt.io/events.source.kind"
+	AnnEventSourceKind = AnnAPIGroup + "/events.source.kind"
 	// AnnEventSource is the source that should be related to events (namespace/name)
-	AnnEventSource = "cdi.kubevirt.io/events.source"
+	AnnEventSource = AnnAPIGroup + "/events.source"
+
+	// AnnAllowClaimAdoption is the annotation that allows a claim to be adopted by a DataVolume
+	AnnAllowClaimAdoption = AnnAPIGroup + "/allowClaimAdoption"
+
+	// AnnCdiCustomizeComponentHash annotation is a hash of all customizations that live under spec.CustomizeComponents
+	AnnCdiCustomizeComponentHash = AnnAPIGroup + "/customizer-identifier"
+
+	// AnnCreatedForDataVolume stores the UID of the datavolume that the PVC was created for
+	AnnCreatedForDataVolume = AnnAPIGroup + "/createdForDataVolume"
 )
 
 // Size-detection pod error codes
@@ -448,6 +464,19 @@ func GetVolumeMode(pvc *corev1.PersistentVolumeClaim) corev1.PersistentVolumeMod
 	return util.ResolveVolumeMode(pvc.Spec.VolumeMode)
 }
 
+// GetStorageClassFromDVSpec returns the StorageClassName from DataVolume PVC or Storage spec
+func GetStorageClassFromDVSpec(dv *cdiv1.DataVolume) *string {
+	if dv.Spec.PVC != nil {
+		return dv.Spec.PVC.StorageClassName
+	}
+
+	if dv.Spec.Storage != nil {
+		return dv.Spec.Storage.StorageClassName
+	}
+
+	return nil
+}
+
 // getStorageClassByName looks up the storage class based on the name.
 // If name is nil, it performs fallback to default according to the provided content type
 // If no storage class is found, returns nil
@@ -485,7 +514,7 @@ func GetStorageClassByNameWithVirtFallback(ctx context.Context, client client.Cl
 	return getStorageClassByName(ctx, client, name, contentType)
 }
 
-// getFallbackStorageClass looks for a default virt/k8s storage class according to the boolean
+// getFallbackStorageClass looks for a default virt/k8s storage class according to the content type
 // If no storage class is found, returns nil
 func getFallbackStorageClass(ctx context.Context, client client.Client, contentType cdiv1.DataVolumeContentType) (*storagev1.StorageClass, error) {
 	storageClasses := &storagev1.StorageClassList{}
@@ -495,16 +524,15 @@ func getFallbackStorageClass(ctx context.Context, client client.Client, contentT
 	}
 
 	if GetContentType(contentType) == cdiv1.DataVolumeKubeVirt {
-		virtSc := GetPlatformDefaultStorageClass(ctx, storageClasses, AnnDefaultVirtStorageClass)
-		if virtSc != nil {
+		if virtSc := GetPlatformDefaultStorageClass(storageClasses, AnnDefaultVirtStorageClass); virtSc != nil {
 			return virtSc, nil
 		}
 	}
-	return GetPlatformDefaultStorageClass(ctx, storageClasses, AnnDefaultStorageClass), nil
+	return GetPlatformDefaultStorageClass(storageClasses, AnnDefaultStorageClass), nil
 }
 
 // GetPlatformDefaultStorageClass returns the default storage class according to the provided annotation or nil if none found
-func GetPlatformDefaultStorageClass(ctx context.Context, storageClasses *storagev1.StorageClassList, defaultAnnotationKey string) *storagev1.StorageClass {
+func GetPlatformDefaultStorageClass(storageClasses *storagev1.StorageClassList, defaultAnnotationKey string) *storagev1.StorageClass {
 	defaultClasses := []storagev1.StorageClass{}
 
 	for _, storageClass := range storageClasses.Items {
@@ -536,13 +564,17 @@ func GetPlatformDefaultStorageClass(ctx context.Context, storageClasses *storage
 
 // GetFilesystemOverheadForStorageClass determines the filesystem overhead defined in CDIConfig for the storageClass.
 func GetFilesystemOverheadForStorageClass(ctx context.Context, client client.Client, storageClassName *string) (cdiv1.Percent, error) {
+	if storageClassName != nil && *storageClassName == "" {
+		klog.V(3).Info("No storage class name passed")
+		return "0", nil
+	}
+
 	cdiConfig := &cdiv1.CDIConfig{}
 	if err := client.Get(ctx, types.NamespacedName{Name: common.ConfigName}, cdiConfig); err != nil {
 		if k8serrors.IsNotFound(err) {
 			klog.V(1).Info("CDIConfig does not exist, pod will not start until it does")
 			return "0", nil
 		}
-
 		return "0", err
 	}
 
@@ -723,6 +755,14 @@ func GetActiveCDI(ctx context.Context, c client.Client) (*cdiv1.CDI, error) {
 		return nil, err
 	}
 
+	if len(crList.Items) == 0 {
+		return nil, nil
+	}
+
+	if len(crList.Items) == 1 {
+		return &crList.Items[0], nil
+	}
+
 	var activeResources []cdiv1.CDI
 	for _, cr := range crList.Items {
 		if cr.Status.Phase != sdkapi.PhaseError {
@@ -730,12 +770,8 @@ func GetActiveCDI(ctx context.Context, c client.Client) (*cdiv1.CDI, error) {
 		}
 	}
 
-	if len(activeResources) == 0 {
-		return nil, nil
-	}
-
-	if len(activeResources) > 1 {
-		return nil, fmt.Errorf("number of active CDI CRs > 1")
+	if len(activeResources) != 1 {
+		return nil, fmt.Errorf("invalid number of active CDI resources: %d", len(activeResources))
 	}
 
 	return &activeResources[0], nil
@@ -1135,9 +1171,9 @@ func SetRestrictedSecurityContext(podSpec *corev1.PodSpec) {
 			container.SecurityContext.SeccompProfile = &corev1.SeccompProfile{
 				Type: corev1.SeccompProfileTypeRuntimeDefault,
 			}
-			container.SecurityContext.AllowPrivilegeEscalation = pointer.Bool(false)
-			container.SecurityContext.RunAsNonRoot = pointer.Bool(true)
-			container.SecurityContext.RunAsUser = pointer.Int64(common.QemuSubGid)
+			container.SecurityContext.AllowPrivilegeEscalation = ptr.To[bool](false)
+			container.SecurityContext.RunAsNonRoot = ptr.To[bool](true)
+			container.SecurityContext.RunAsUser = ptr.To[int64](common.QemuSubGid)
 			if len(container.VolumeMounts) > 0 {
 				hasVolumeMounts = true
 			}
@@ -1148,7 +1184,7 @@ func SetRestrictedSecurityContext(podSpec *corev1.PodSpec) {
 		if podSpec.SecurityContext == nil {
 			podSpec.SecurityContext = &corev1.PodSecurityContext{}
 		}
-		podSpec.SecurityContext.FSGroup = pointer.Int64(common.QemuSubGid)
+		podSpec.SecurityContext.FSGroup = ptr.To[int64](common.QemuSubGid)
 	}
 }
 
@@ -1190,6 +1226,9 @@ func CreatePvcInStorageClass(name, ns string, storageClassName *string, annotati
 		},
 	}
 	pvc.Status.Capacity = pvc.Spec.Resources.Requests.DeepCopy()
+	if pvc.Status.Phase == corev1.ClaimBound {
+		pvc.Spec.VolumeName = "pv-" + string(pvc.UID)
+	}
 	return pvc
 }
 
@@ -2068,10 +2107,11 @@ func OwnedByDataVolume(obj metav1.Object) bool {
 // SetPvcAllowedAnnotations applies PVC annotations on the given obj
 func SetPvcAllowedAnnotations(obj metav1.Object, pvc *corev1.PersistentVolumeClaim) {
 	allowedAnnotations := map[string]string{
-		AnnPodNetwork:              "",
-		AnnPodSidecarInjection:     AnnPodSidecarInjectionDefault,
-		AnnPriorityClassName:       "",
-		AnnPodMultusDefaultNetwork: ""}
+		AnnPodNetwork:                 "",
+		AnnPodSidecarInjectionIstio:   AnnPodSidecarInjectionIstioDefault,
+		AnnPodSidecarInjectionLinkerd: AnnPodSidecarInjectionLinkerdDefault,
+		AnnPriorityClassName:          "",
+		AnnPodMultusDefaultNetwork:    ""}
 	for ann, def := range allowedAnnotations {
 		val, ok := pvc.Annotations[ann]
 		if !ok && def != "" {
@@ -2083,3 +2123,34 @@ func SetPvcAllowedAnnotations(obj metav1.Object, pvc *corev1.PersistentVolumeCla
 		}
 	}
 }
+
+// ClaimMayExistBeforeDataVolume returns true if the PVC may exist before the DataVolume
+func ClaimMayExistBeforeDataVolume(c client.Client, pvc *corev1.PersistentVolumeClaim, dv *cdiv1.DataVolume) (bool, error) {
+	if ClaimIsPopulatedForDataVolume(pvc, dv) {
+		return true, nil
+	}
+	return AllowClaimAdoption(c, pvc, dv)
+}
+
+// ClaimIsPopulatedForDataVolume returns true if the PVC is populated for the given DataVolume
+func ClaimIsPopulatedForDataVolume(pvc *corev1.PersistentVolumeClaim, dv *cdiv1.DataVolume) bool {
+	return pvc != nil && dv != nil && pvc.Annotations[AnnPopulatedFor] == dv.Name
+}
+
+// AllowClaimAdoption returns true if the PVC may be adopted
+func AllowClaimAdoption(c client.Client, pvc *corev1.PersistentVolumeClaim, dv *cdiv1.DataVolume) (bool, error) {
+	if pvc == nil || dv == nil {
+		return false, nil
+	}
+	anno, ok := pvc.Annotations[AnnCreatedForDataVolume]
+	if ok && anno == string(dv.UID) {
+		return false, nil
+	}
+	anno, ok = dv.Annotations[AnnAllowClaimAdoption]
+	// if annotation exists, go with that regardless of featuregate
+	if ok {
+		val, _ := strconv.ParseBool(anno)
+		return val, nil
+	}
+	return featuregates.NewFeatureGates(c).ClaimAdoptionEnabled()
+}
