@@ -27,10 +27,9 @@ import (
 
 	"github.com/containers/image/v5/docker/reference"
 	"github.com/go-logr/logr"
-	"github.com/gorhill/cronexpr"
 	imagev1 "github.com/openshift/api/image/v1"
 	"github.com/pkg/errors"
-	"github.com/prometheus/client_golang/prometheus"
+	cronexpr "github.com/robfig/cron/v3"
 
 	snapshotv1 "github.com/kubernetes-csi/external-snapshotter/client/v6/apis/volumesnapshot/v1"
 	batchv1 "k8s.io/api/batch/v1"
@@ -42,9 +41,8 @@ import (
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/apimachinery/pkg/util/validation"
 	"k8s.io/client-go/tools/record"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/event"
@@ -58,7 +56,7 @@ import (
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
 	cdv "kubevirt.io/containerized-data-importer/pkg/controller/datavolume"
-	"kubevirt.io/containerized-data-importer/pkg/monitoring"
+	metrics "kubevirt.io/containerized-data-importer/pkg/monitoring/metrics/cdi-controller"
 	"kubevirt.io/containerized-data-importer/pkg/operator"
 	"kubevirt.io/containerized-data-importer/pkg/util"
 	"kubevirt.io/containerized-data-importer/pkg/util/naming"
@@ -74,17 +72,6 @@ const (
 	prometheusCronNameLabel = "cron_name"
 )
 
-var (
-	// DataImportCronOutdatedGauge is the metric we use to alert about DataImportCrons failing
-	DataImportCronOutdatedGauge = prometheus.NewGaugeVec(
-		prometheus.GaugeOpts{
-			Name: monitoring.MetricOptsList[monitoring.DataImportCronOutdated].Name,
-			Help: monitoring.MetricOptsList[monitoring.DataImportCronOutdated].Help,
-		},
-		[]string{prometheusNsLabel, prometheusCronNameLabel},
-	)
-)
-
 // DataImportCronReconciler members
 type DataImportCronReconciler struct {
 	client          client.Client
@@ -276,7 +263,7 @@ func (r *DataImportCronReconciler) pollImageStreamDigest(ctx context.Context, da
 
 func (r *DataImportCronReconciler) setNextCronTime(dataImportCron *cdiv1.DataImportCron) (reconcile.Result, error) {
 	now := time.Now()
-	expr, err := cronexpr.Parse(dataImportCron.Spec.Schedule)
+	expr, err := cronexpr.ParseStandard(dataImportCron.Spec.Schedule)
 	if err != nil {
 		return reconcile.Result{}, err
 	}
@@ -318,7 +305,7 @@ func (r *DataImportCronReconciler) update(ctx context.Context, dataImportCron *c
 	importSucceeded := false
 
 	dataVolume := dataImportCron.Spec.Template
-	explicitScName := getStorageClassFromTemplate(&dataVolume)
+	explicitScName := cc.GetStorageClassFromDVSpec(&dataVolume)
 	desiredStorageClass, err := cc.GetStorageClassByNameWithVirtFallback(ctx, r.client, explicitScName, dataVolume.Spec.ContentType)
 	if err != nil {
 		return res, err
@@ -375,7 +362,8 @@ func (r *DataImportCronReconciler) update(ctx context.Context, dataImportCron *c
 		importSucceeded = true
 	} else {
 		if len(imports) > 0 {
-			dataImportCron.Status.CurrentImports = imports[1:]
+			imports = imports[1:]
+			dataImportCron.Status.CurrentImports = imports
 		}
 		updateDataImportCronCondition(dataImportCron, cdiv1.DataImportCronProgressing, corev1.ConditionFalse, "No current import", noImport)
 	}
@@ -777,18 +765,6 @@ func (r *DataImportCronReconciler) getSourceFormat(ctx context.Context, desiredS
 	return format, nil
 }
 
-func getStorageClassFromTemplate(dataVolume *cdiv1.DataVolume) *string {
-	if dataVolume.Spec.PVC != nil {
-		return dataVolume.Spec.PVC.StorageClassName
-	}
-
-	if dataVolume.Spec.Storage != nil {
-		return dataVolume.Spec.Storage.StorageClassName
-	}
-
-	return nil
-}
-
 func (r *DataImportCronReconciler) garbageCollectOldImports(ctx context.Context, cron *cdiv1.DataImportCron) error {
 	if cron.Spec.GarbageCollect != nil && *cron.Spec.GarbageCollect != cdiv1.DataImportCronGarbageCollectOutdated {
 		return nil
@@ -896,7 +872,7 @@ func (r *DataImportCronReconciler) garbageCollectSnapshots(ctx context.Context,
 
 func (r *DataImportCronReconciler) cleanup(ctx context.Context, cron types.NamespacedName) error {
 	// Don't keep alerting over a cron thats being deleted, will get set back to 1 again by reconcile loop if needed.
-	DataImportCronOutdatedGauge.With(getPrometheusCronLabels(cron)).Set(0)
+	metrics.DeleteDataImportCronOutdated(getPrometheusCronLabels(cron))
 	if err := r.deleteJobs(ctx, cron); err != nil {
 		return err
 	}
@@ -921,30 +897,19 @@ func (r *DataImportCronReconciler) cleanup(ctx context.Context, cron types.Names
 }
 
 func (r *DataImportCronReconciler) deleteJobs(ctx context.Context, cron types.NamespacedName) error {
-	deletePropagationBackground := metav1.DeletePropagationBackground
-	deleteOpts := &client.DeleteOptions{PropagationPolicy: &deletePropagationBackground}
-	selector, err := getSelector(map[string]string{common.DataImportCronLabel: getCronJobLabelValue(cron.Namespace, cron.Name)})
+	deleteOpts := client.DeleteOptions{PropagationPolicy: ptr.To[metav1.DeletionPropagation](metav1.DeletePropagationBackground)}
+	selector, err := getSelector(map[string]string{common.DataImportCronNsLabel: cron.Namespace, common.DataImportCronLabel: cron.Name})
 	if err != nil {
 		return err
 	}
-	cronJobList := &batchv1.CronJobList{}
-	if err := r.client.List(ctx, cronJobList, &client.ListOptions{Namespace: r.cdiNamespace, LabelSelector: selector}); err != nil {
+	opts := &client.DeleteAllOfOptions{ListOptions: client.ListOptions{Namespace: r.cdiNamespace, LabelSelector: selector}, DeleteOptions: deleteOpts}
+	if err := r.client.DeleteAllOf(ctx, &batchv1.CronJob{}, opts); err != nil {
 		return err
 	}
-	for _, cronJob := range cronJobList.Items {
-		if err := r.client.Delete(ctx, &cronJob, deleteOpts); cc.IgnoreNotFound(err) != nil {
-			return err
-		}
-	}
-	jobList := &batchv1.JobList{}
-	if err := r.client.List(ctx, jobList, &client.ListOptions{Namespace: r.cdiNamespace, LabelSelector: selector}); err != nil {
+	if err := r.client.DeleteAllOf(ctx, &batchv1.Job{}, opts); err != nil {
 		return err
 	}
-	for _, job := range jobList.Items {
-		if err := r.client.Delete(ctx, &job, deleteOpts); cc.IgnoreNotFound(err) != nil {
-			return err
-		}
-	}
+
 	return nil
 }
 
@@ -983,25 +948,28 @@ func NewDataImportCronController(mgr manager.Manager, log logr.Logger, importerI
 }
 
 func addDataImportCronControllerWatches(mgr manager.Manager, c controller.Controller, log logr.Logger) error {
-	if err := c.Watch(&source.Kind{Type: &cdiv1.DataImportCron{}}, &handler.EnqueueRequestForObject{}); err != nil {
+	if err := c.Watch(source.Kind(mgr.GetCache(), &cdiv1.DataImportCron{}), &handler.EnqueueRequestForObject{}); err != nil {
 		return err
 	}
 
 	getCronName := func(obj client.Object) string {
 		return obj.GetLabels()[common.DataImportCronLabel]
 	}
-	mapSourceObjectToCron := func(obj client.Object) []reconcile.Request {
+	getCronNs := func(obj client.Object) string {
+		return obj.GetLabels()[common.DataImportCronNsLabel]
+	}
+	mapSourceObjectToCron := func(_ context.Context, obj client.Object) []reconcile.Request {
 		if cronName := getCronName(obj); cronName != "" {
 			return []reconcile.Request{{NamespacedName: types.NamespacedName{Name: cronName, Namespace: obj.GetNamespace()}}}
 		}
 		return nil
 	}
 
-	mapStorageProfileToCron := func(obj client.Object) (reqs []reconcile.Request) {
+	mapStorageProfileToCron := func(ctx context.Context, obj client.Object) (reqs []reconcile.Request) {
 		// TODO: Get rid of this after at least one version; use indexer on storage class annotation instead
 		// Otherwise we risk losing the storage profile event
 		var crons cdiv1.DataImportCronList
-		if err := mgr.GetClient().List(context.TODO(), &crons); err != nil {
+		if err := mgr.GetClient().List(ctx, &crons); err != nil {
 			c.GetLogger().Error(err, "Unable to list DataImportCrons")
 			return
 		}
@@ -1009,8 +977,8 @@ func addDataImportCronControllerWatches(mgr manager.Manager, c controller.Contro
 		scName := obj.GetName()
 		for _, cron := range crons.Items {
 			dataVolume := cron.Spec.Template
-			explicitScName := getStorageClassFromTemplate(&dataVolume)
-			templateSc, err := cc.GetStorageClassByNameWithVirtFallback(context.TODO(), mgr.GetClient(), explicitScName, dataVolume.Spec.ContentType)
+			explicitScName := cc.GetStorageClassFromDVSpec(&dataVolume)
+			templateSc, err := cc.GetStorageClassByNameWithVirtFallback(ctx, mgr.GetClient(), explicitScName, dataVolume.Spec.ContentType)
 			if err != nil || templateSc == nil {
 				c.GetLogger().Error(err, "Unable to get storage class", "templateSc", templateSc)
 				return
@@ -1022,7 +990,7 @@ func addDataImportCronControllerWatches(mgr manager.Manager, c controller.Contro
 		return
 	}
 
-	if err := c.Watch(&source.Kind{Type: &cdiv1.DataVolume{}},
+	if err := c.Watch(source.Kind(mgr.GetCache(), &cdiv1.DataVolume{}),
 		handler.EnqueueRequestsFromMapFunc(mapSourceObjectToCron),
 		predicate.Funcs{
 			CreateFunc: func(event.CreateEvent) bool { return false },
@@ -1033,7 +1001,7 @@ func addDataImportCronControllerWatches(mgr manager.Manager, c controller.Contro
 		return err
 	}
 
-	if err := c.Watch(&source.Kind{Type: &cdiv1.DataSource{}},
+	if err := c.Watch(source.Kind(mgr.GetCache(), &cdiv1.DataSource{}),
 		handler.EnqueueRequestsFromMapFunc(mapSourceObjectToCron),
 		predicate.Funcs{
 			CreateFunc: func(event.CreateEvent) bool { return false },
@@ -1044,7 +1012,7 @@ func addDataImportCronControllerWatches(mgr manager.Manager, c controller.Contro
 		return err
 	}
 
-	if err := c.Watch(&source.Kind{Type: &cdiv1.StorageProfile{}},
+	if err := c.Watch(source.Kind(mgr.GetCache(), &cdiv1.StorageProfile{}),
 		handler.EnqueueRequestsFromMapFunc(mapStorageProfileToCron),
 		predicate.Funcs{
 			CreateFunc: func(event.CreateEvent) bool { return true },
@@ -1059,6 +1027,21 @@ func addDataImportCronControllerWatches(mgr manager.Manager, c controller.Contro
 		return err
 	}
 
+	mapCronJobToCron := func(_ context.Context, obj client.Object) []reconcile.Request {
+		return []reconcile.Request{{NamespacedName: types.NamespacedName{Namespace: getCronNs(obj), Name: getCronName(obj)}}}
+	}
+
+	if err := c.Watch(source.Kind(mgr.GetCache(), &batchv1.CronJob{}),
+		handler.EnqueueRequestsFromMapFunc(mapCronJobToCron),
+		predicate.Funcs{
+			CreateFunc: func(e event.CreateEvent) bool { return getCronName(e.Object) != "" && getCronNs(e.Object) != "" },
+			DeleteFunc: func(event.DeleteEvent) bool { return false },
+			UpdateFunc: func(event.UpdateEvent) bool { return false },
+		},
+	); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -1203,7 +1186,7 @@ func InitPollerPodSpec(c client.Client, cron *cdiv1.DataImportCron, podSpec *cor
 	}
 
 	podSpec.RestartPolicy = corev1.RestartPolicyNever
-	podSpec.TerminationGracePeriodSeconds = pointer.Int64(0)
+	podSpec.TerminationGracePeriodSeconds = ptr.To[int64](0)
 	podSpec.Containers = []corev1.Container{container}
 	podSpec.ServiceAccountName = common.CronJobServiceAccountName
 	podSpec.Volumes = volumes
@@ -1221,12 +1204,12 @@ func (r *DataImportCronReconciler) initCronJob(cron *cdiv1.DataImportCron, cronJ
 	cronJobSpec := &cronJob.Spec
 	cronJobSpec.Schedule = cron.Spec.Schedule
 	cronJobSpec.ConcurrencyPolicy = batchv1.ForbidConcurrent
-	cronJobSpec.SuccessfulJobsHistoryLimit = pointer.Int32(1)
-	cronJobSpec.FailedJobsHistoryLimit = pointer.Int32(1)
+	cronJobSpec.SuccessfulJobsHistoryLimit = ptr.To[int32](1)
+	cronJobSpec.FailedJobsHistoryLimit = ptr.To[int32](1)
 
 	jobSpec := &cronJobSpec.JobTemplate.Spec
-	jobSpec.BackoffLimit = pointer.Int32(2)
-	jobSpec.TTLSecondsAfterFinished = pointer.Int32(10)
+	jobSpec.BackoffLimit = ptr.To[int32](2)
+	jobSpec.TTLSecondsAfterFinished = ptr.To[int32](10)
 
 	podSpec := &jobSpec.Template.Spec
 	if err := InitPollerPodSpec(r.client, cron, podSpec, r.image, corev1.PullPolicy(r.pullPolicy), r.log); err != nil {
@@ -1258,7 +1241,8 @@ func (r *DataImportCronReconciler) setJobCommon(cron *cdiv1.DataImportCron, obj
 	}
 	util.SetRecommendedLabels(obj, r.installerLabels, common.CDIControllerName)
 	labels := obj.GetLabels()
-	labels[common.DataImportCronLabel] = getCronJobLabelValue(cron.Namespace, cron.Name)
+	labels[common.DataImportCronNsLabel] = cron.Namespace
+	labels[common.DataImportCronLabel] = cron.Name
 	obj.SetLabels(labels)
 	return nil
 }
@@ -1278,6 +1262,7 @@ func (r *DataImportCronReconciler) newSourceDataVolume(cron *cdiv1.DataImportCro
 	dv.Namespace = cron.Namespace
 	r.setDataImportCronResourceLabels(cron, dv)
 	cc.AddAnnotation(dv, cc.AnnImmediateBinding, "true")
+	cc.AddAnnotation(dv, AnnLastUseTime, time.Now().UTC().Format(time.RFC3339Nano))
 	passCronAnnotationToDv(cron, dv, cc.AnnPodRetainAfterCompletion)
 
 	for _, defaultInstanceTypeLabel := range cc.DefaultInstanceTypeLabels {
@@ -1369,12 +1354,3 @@ func GetInitialJobName(cron *cdiv1.DataImportCron) string {
 func getSelector(matchLabels map[string]string) (labels.Selector, error) {
 	return metav1.LabelSelectorAsSelector(&metav1.LabelSelector{MatchLabels: matchLabels})
 }
-
-func getCronJobLabelValue(cronNamespace, cronName string) string {
-	const maxLen = validation.DNS1035LabelMaxLength
-	label := cronNamespace + "." + cronName
-	if len(label) > maxLen {
-		return label[:maxLen]
-	}
-	return label
-}
