@@ -40,7 +40,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
@@ -251,9 +251,7 @@ var _ = Describe("All DataImportCron Tests", func() {
 			err = reconciler.client.Get(context.TODO(), cronKey, cron)
 			Expect(err).ToNot(HaveOccurred())
 
-			now := metav1.Now()
-			cron.DeletionTimestamp = &now
-			err = reconciler.client.Update(context.TODO(), cron)
+			err = reconciler.client.Delete(context.TODO(), cron)
 			Expect(err).ToNot(HaveOccurred())
 
 			_, err = reconciler.Reconcile(context.TODO(), cronReq)
@@ -263,6 +261,60 @@ var _ = Describe("All DataImportCron Tests", func() {
 			Expect(err).To(HaveOccurred())
 		})
 
+		It("Should delete DataImportCron-orphan CronJob", func() {
+			reconciler = createDataImportCronReconciler()
+
+			cron = newDataImportCron(cronName)
+			err := reconciler.client.Create(context.TODO(), cron)
+			Expect(err).ToNot(HaveOccurred())
+
+			_, err = reconciler.Reconcile(context.TODO(), cronReq)
+			Expect(err).ToNot(HaveOccurred())
+			cronjob := &batchv1.CronJob{}
+			err = reconciler.client.Get(context.TODO(), cronJobKey(cron), cronjob)
+			Expect(err).ToNot(HaveOccurred())
+
+			err = reconciler.client.Delete(context.TODO(), cron)
+			Expect(err).ToNot(HaveOccurred())
+
+			_, err = reconciler.Reconcile(context.TODO(), cronReq)
+			Expect(err).ToNot(HaveOccurred())
+			err = reconciler.client.Get(context.TODO(), cronJobKey(cron), cronjob)
+			Expect(err).To(HaveOccurred())
+			Expect(k8serrors.IsNotFound(err)).To(BeTrue())
+		})
+
+		It("Should update CronJob DataImportCron labels", func() {
+			reconciler = createDataImportCronReconciler()
+
+			cron = newDataImportCron(cronName)
+			err := reconciler.client.Create(context.TODO(), cron)
+			Expect(err).ToNot(HaveOccurred())
+
+			_, err = reconciler.Reconcile(context.TODO(), cronReq)
+			Expect(err).ToNot(HaveOccurred())
+			cronjob := &batchv1.CronJob{}
+			err = reconciler.client.Get(context.TODO(), cronJobKey(cron), cronjob)
+			Expect(err).ToNot(HaveOccurred())
+
+			cronjob.Labels = nil
+			err = reconciler.client.Update(context.TODO(), cronjob)
+			Expect(err).ToNot(HaveOccurred())
+
+			_, err = reconciler.Reconcile(context.TODO(), cronReq)
+			Expect(err).ToNot(HaveOccurred())
+			err = reconciler.client.Get(context.TODO(), cronJobKey(cron), cronjob)
+			Expect(err).ToNot(HaveOccurred())
+
+			cronNsLabel, ok := cronjob.Labels[common.DataImportCronNsLabel]
+			Expect(ok).To(BeTrue())
+			Expect(cronNsLabel).To(Equal(cron.Namespace))
+
+			cronLabel, ok := cronjob.Labels[common.DataImportCronLabel]
+			Expect(ok).To(BeTrue())
+			Expect(cronLabel).To(Equal(cron.Name))
+		})
+
 		It("Should verify CronJob container env variables are empty and no extra volume is set when proxy is not configured", func() {
 			cron = newDataImportCron(cronName)
 			reconciler = createDataImportCronReconciler(cron)
@@ -273,11 +325,11 @@ var _ = Describe("All DataImportCron Tests", func() {
 			err = reconciler.client.Get(context.TODO(), cronJobKey(cron), cronjob)
 			Expect(err).ToNot(HaveOccurred())
 
-			Expect(cronjob.Spec.SuccessfulJobsHistoryLimit).To(Equal(pointer.Int32(1)))
-			Expect(cronjob.Spec.FailedJobsHistoryLimit).To(Equal(pointer.Int32(1)))
+			Expect(cronjob.Spec.SuccessfulJobsHistoryLimit).To(Equal(ptr.To[int32](1)))
+			Expect(cronjob.Spec.FailedJobsHistoryLimit).To(Equal(ptr.To[int32](1)))
 
 			jobTemplateSpec := cronjob.Spec.JobTemplate.Spec
-			Expect(jobTemplateSpec.TTLSecondsAfterFinished).To(Equal(pointer.Int32(10)))
+			Expect(jobTemplateSpec.TTLSecondsAfterFinished).To(Equal(ptr.To[int32](10)))
 
 			jobPodTemplateSpec := jobTemplateSpec.Template.Spec
 			containers := jobPodTemplateSpec.Containers
@@ -445,9 +497,7 @@ var _ = Describe("All DataImportCron Tests", func() {
 			Expect(*cron.Status.LastImportedPVC).To(Equal(sourcePVC))
 			Expect(cron.Status.LastImportTimestamp).ToNot(BeNil())
 
-			now := metav1.Now()
-			cron.DeletionTimestamp = &now
-			err = reconciler.client.Update(context.TODO(), cron)
+			err = reconciler.client.Delete(context.TODO(), cron)
 			Expect(err).ToNot(HaveOccurred())
 			_, err = reconciler.Reconcile(context.TODO(), cronReq)
 			Expect(err).ToNot(HaveOccurred())
@@ -464,6 +514,38 @@ var _ = Describe("All DataImportCron Tests", func() {
 			Entry("empty schedule", emptySchedule, "should succeed with an empty schedule"),
 		)
 
+		It("Should recreate DataVolume if the last import was deleted", func() {
+			cron = newDataImportCron(cronName)
+			cron.Annotations[AnnSourceDesiredDigest] = testDigest
+			reconciler = createDataImportCronReconciler(cron)
+
+			_, err := reconciler.Reconcile(context.TODO(), cronReq)
+			Expect(err).ToNot(HaveOccurred())
+			err = reconciler.client.Get(context.TODO(), cronKey, cron)
+			Expect(err).ToNot(HaveOccurred())
+
+			imports := cron.Status.CurrentImports
+			Expect(imports).ToNot(BeEmpty())
+			dvName := imports[0].DataVolumeName
+			Expect(dvName).ToNot(BeEmpty())
+
+			dv := &cdiv1.DataVolume{}
+			err = reconciler.client.Get(context.TODO(), dvKey(dvName), dv)
+			Expect(err).ToNot(HaveOccurred())
+
+			err = reconciler.client.Delete(context.TODO(), dv)
+			Expect(err).ToNot(HaveOccurred())
+			err = reconciler.client.Get(context.TODO(), dvKey(dvName), dv)
+			Expect(err).To(HaveOccurred())
+			Expect(k8serrors.IsNotFound(err)).To(BeTrue())
+
+			_, err = reconciler.Reconcile(context.TODO(), cronReq)
+			Expect(err).ToNot(HaveOccurred())
+
+			err = reconciler.client.Get(context.TODO(), dvKey(dvName), dv)
+			Expect(err).ToNot(HaveOccurred())
+		})
+
 		It("Should not create DV if PVC exists on DesiredDigest update; Should update DIC and DAS, and GC LRU PVCs", func() {
 			const nPVCs = 3
 			var (
@@ -663,9 +745,7 @@ var _ = Describe("All DataImportCron Tests", func() {
 			Expect(err).ToNot(HaveOccurred())
 			verifyConditions("Import succeeded", false, true, true, noImport, upToDate, ready, &corev1.PersistentVolumeClaim{})
 
-			now := metav1.Now()
-			cron.DeletionTimestamp = &now
-			err = reconciler.client.Update(context.TODO(), cron)
+			err = reconciler.client.Delete(context.TODO(), cron)
 			Expect(err).ToNot(HaveOccurred())
 			_, err = reconciler.Reconcile(context.TODO(), cronReq)
 			Expect(err).ToNot(HaveOccurred())
@@ -903,7 +983,7 @@ var _ = Describe("All DataImportCron Tests", func() {
 				err = reconciler.client.Get(context.TODO(), dvKey(dvName), snap)
 				Expect(err).ToNot(HaveOccurred())
 				snap.Status = &snapshotv1.VolumeSnapshotStatus{
-					ReadyToUse: pointer.Bool(true),
+					ReadyToUse: ptr.To[bool](true),
 				}
 				err = reconciler.client.Update(context.TODO(), snap)
 				Expect(err).ToNot(HaveOccurred())
@@ -935,9 +1015,7 @@ var _ = Describe("All DataImportCron Tests", func() {
 				Expect(*snap.Status.ReadyToUse).To(BeTrue())
 				Expect(*snap.Spec.Source.PersistentVolumeClaimName).To(Equal(dvName))
 
-				now := metav1.Now()
-				cron.DeletionTimestamp = &now
-				err = reconciler.client.Update(context.TODO(), cron)
+				err = reconciler.client.Delete(context.TODO(), cron)
 				Expect(err).ToNot(HaveOccurred())
 				_, err = reconciler.Reconcile(context.TODO(), cronReq)
 				Expect(err).ToNot(HaveOccurred())
