@@ -38,6 +38,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/record"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
@@ -50,6 +51,7 @@ import (
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
 	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
+	metrics "kubevirt.io/containerized-data-importer/pkg/monitoring/metrics/cdi-controller"
 	"kubevirt.io/containerized-data-importer/pkg/token"
 	"kubevirt.io/containerized-data-importer/pkg/util"
 )
@@ -76,7 +78,13 @@ const (
 	claimStorageClassNameField = "spec.storageClassName"
 )
 
-var httpClient *http.Client
+var (
+	httpClient *http.Client
+
+	delayedAnnotations = []string{
+		cc.AnnPopulatedFor,
+	}
+)
 
 // Event represents DV controller event
 type Event struct {
@@ -116,7 +124,7 @@ type ReconcilerBase struct {
 	shouldUpdateProgress bool
 }
 
-func pvcIsPopulated(pvc *corev1.PersistentVolumeClaim, dv *cdiv1.DataVolume) bool {
+func pvcIsPopulatedForDataVolume(pvc *corev1.PersistentVolumeClaim, dv *cdiv1.DataVolume) bool {
 	if pvc == nil || dv == nil {
 		return false
 	}
@@ -192,14 +200,9 @@ func getIndexArgs() []indexArgs {
 			},
 		},
 		{
-			obj:   &corev1.PersistentVolume{},
-			field: claimStorageClassNameField,
-			extractValue: func(obj client.Object) []string {
-				if pv, ok := obj.(*corev1.PersistentVolume); ok && pv.Status.Phase == corev1.VolumeAvailable {
-					return []string{pv.Spec.StorageClassName}
-				}
-				return nil
-			},
+			obj:          &corev1.PersistentVolume{},
+			field:        claimStorageClassNameField,
+			extractValue: extractAvailablePersistentVolumeStorageClassName,
 		},
 	}
 }
@@ -218,44 +221,58 @@ func CreateCommonIndexes(mgr manager.Manager) error {
 	return nil
 }
 
+// CreateAvailablePersistentVolumeIndex adds storage class name index for available PersistentVolumes
+func CreateAvailablePersistentVolumeIndex(fieldIndexer client.FieldIndexer) error {
+	return fieldIndexer.IndexField(context.TODO(), &corev1.PersistentVolume{},
+		claimStorageClassNameField, extractAvailablePersistentVolumeStorageClassName)
+}
+
+func extractAvailablePersistentVolumeStorageClassName(obj client.Object) []string {
+	if pv, ok := obj.(*corev1.PersistentVolume); ok && pv.Status.Phase == corev1.VolumeAvailable {
+		return []string{pv.Spec.StorageClassName}
+	}
+	return nil
+}
+
 func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeController controller.Controller, op dataVolumeOp) error {
-	appendMatchingDataVolumeRequest := func(reqs []reconcile.Request, mgr manager.Manager, namespace, name string) []reconcile.Request {
+	appendMatchingDataVolumeRequest := func(ctx context.Context, reqs []reconcile.Request, mgr manager.Manager, namespace, name string) []reconcile.Request {
 		dvKey := types.NamespacedName{Namespace: namespace, Name: name}
 		dv := &cdiv1.DataVolume{}
-		if err := mgr.GetClient().Get(context.TODO(), dvKey, dv); err != nil {
+		if err := mgr.GetClient().Get(ctx, dvKey, dv); err != nil {
 			if !k8serrors.IsNotFound(err) {
 				mgr.GetLogger().Error(err, "Failed to get DV", "dvKey", dvKey)
 			}
 			return reqs
 		}
-		if getDataVolumeOp(mgr.GetLogger(), dv, mgr.GetClient()) == op {
+		if getDataVolumeOp(ctx, mgr.GetLogger(), dv, mgr.GetClient()) == op {
 			reqs = append(reqs, reconcile.Request{NamespacedName: dvKey})
 		}
 		return reqs
 	}
 
 	// Setup watches
-	if err := dataVolumeController.Watch(&source.Kind{Type: &cdiv1.DataVolume{}}, handler.EnqueueRequestsFromMapFunc(
-		func(obj client.Object) []reconcile.Request {
+	if err := dataVolumeController.Watch(source.Kind(mgr.GetCache(), &cdiv1.DataVolume{}), handler.EnqueueRequestsFromMapFunc(
+		func(ctx context.Context, obj client.Object) []reconcile.Request {
 			dv := obj.(*cdiv1.DataVolume)
-			if getDataVolumeOp(mgr.GetLogger(), dv, mgr.GetClient()) != op {
+			if getDataVolumeOp(ctx, mgr.GetLogger(), dv, mgr.GetClient()) != op {
 				return nil
 			}
+			updatePendingDataVolumesGauge(ctx, mgr.GetLogger(), dv, mgr.GetClient())
 			return []reconcile.Request{{NamespacedName: types.NamespacedName{Namespace: dv.Namespace, Name: dv.Name}}}
 		}),
 	); err != nil {
 		return err
 	}
-	if err := dataVolumeController.Watch(&source.Kind{Type: &corev1.PersistentVolumeClaim{}}, handler.EnqueueRequestsFromMapFunc(
-		func(obj client.Object) []reconcile.Request {
+	if err := dataVolumeController.Watch(source.Kind(mgr.GetCache(), &corev1.PersistentVolumeClaim{}), handler.EnqueueRequestsFromMapFunc(
+		func(ctx context.Context, obj client.Object) []reconcile.Request {
 			var result []reconcile.Request
 			owner := metav1.GetControllerOf(obj)
 			if owner != nil && owner.Kind == "DataVolume" {
-				result = appendMatchingDataVolumeRequest(result, mgr, obj.GetNamespace(), owner.Name)
+				result = appendMatchingDataVolumeRequest(ctx, result, mgr, obj.GetNamespace(), owner.Name)
 			}
 			populatedFor := obj.GetAnnotations()[cc.AnnPopulatedFor]
 			if populatedFor != "" {
-				result = appendMatchingDataVolumeRequest(result, mgr, obj.GetNamespace(), populatedFor)
+				result = appendMatchingDataVolumeRequest(ctx, result, mgr, obj.GetNamespace(), populatedFor)
 			}
 			// it is okay if result contains the same entry twice, will be deduplicated by caller
 			return result
@@ -263,20 +280,20 @@ func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeControl
 	); err != nil {
 		return err
 	}
-	if err := dataVolumeController.Watch(&source.Kind{Type: &corev1.Pod{}}, handler.EnqueueRequestsFromMapFunc(
-		func(obj client.Object) []reconcile.Request {
+	if err := dataVolumeController.Watch(source.Kind(mgr.GetCache(), &corev1.Pod{}), handler.EnqueueRequestsFromMapFunc(
+		func(ctx context.Context, obj client.Object) []reconcile.Request {
 			owner := metav1.GetControllerOf(obj)
 			if owner == nil || owner.Kind != "DataVolume" {
 				return nil
 			}
-			return appendMatchingDataVolumeRequest(nil, mgr, obj.GetNamespace(), owner.Name)
+			return appendMatchingDataVolumeRequest(ctx, nil, mgr, obj.GetNamespace(), owner.Name)
 		}),
 	); err != nil {
 		return err
 	}
 	for _, k := range []client.Object{&corev1.PersistentVolumeClaim{}, &corev1.Pod{}, &cdiv1.ObjectTransfer{}} {
-		if err := dataVolumeController.Watch(&source.Kind{Type: k}, handler.EnqueueRequestsFromMapFunc(
-			func(obj client.Object) []reconcile.Request {
+		if err := dataVolumeController.Watch(source.Kind(mgr.GetCache(), k), handler.EnqueueRequestsFromMapFunc(
+			func(ctx context.Context, obj client.Object) []reconcile.Request {
 				if !hasAnnOwnedByDataVolume(obj) {
 					return nil
 				}
@@ -284,7 +301,7 @@ func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeControl
 				if err != nil {
 					return nil
 				}
-				return appendMatchingDataVolumeRequest(nil, mgr, namespace, name)
+				return appendMatchingDataVolumeRequest(ctx, nil, mgr, namespace, name)
 			}),
 		); err != nil {
 			return err
@@ -292,14 +309,15 @@ func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeControl
 	}
 
 	// Watch for SC updates and reconcile the DVs waiting for default SC
-	if err := dataVolumeController.Watch(&source.Kind{Type: &storagev1.StorageClass{}}, handler.EnqueueRequestsFromMapFunc(
-		func(obj client.Object) (reqs []reconcile.Request) {
+	// Relevant only when the DV StorageSpec has no AccessModes set and no matching StorageClass yet, so PVC cannot be created (test_id:9922)
+	if err := dataVolumeController.Watch(source.Kind(mgr.GetCache(), &storagev1.StorageClass{}), handler.EnqueueRequestsFromMapFunc(
+		func(ctx context.Context, obj client.Object) (reqs []reconcile.Request) {
 			dvList := &cdiv1.DataVolumeList{}
-			if err := mgr.GetClient().List(context.TODO(), dvList, client.MatchingFields{dvPhaseField: ""}); err != nil {
+			if err := mgr.GetClient().List(ctx, dvList, client.MatchingFields{dvPhaseField: ""}); err != nil {
 				return
 			}
 			for _, dv := range dvList.Items {
-				if getDataVolumeOp(mgr.GetLogger(), &dv, mgr.GetClient()) == op {
+				if getDataVolumeOp(ctx, mgr.GetLogger(), &dv, mgr.GetClient()) == op {
 					reqs = append(reqs, reconcile.Request{NamespacedName: types.NamespacedName{Name: dv.Name, Namespace: dv.Namespace}})
 				}
 			}
@@ -311,11 +329,12 @@ func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeControl
 	}
 
 	// Watch for PV updates to reconcile the DVs waiting for available PV
-	if err := dataVolumeController.Watch(&source.Kind{Type: &corev1.PersistentVolume{}}, handler.EnqueueRequestsFromMapFunc(
-		func(obj client.Object) (reqs []reconcile.Request) {
+	// Relevant only when the DV StorageSpec has no AccessModes set and no matching StorageClass yet, so PVC cannot be created (test_id:9924,9925)
+	if err := dataVolumeController.Watch(source.Kind(mgr.GetCache(), &corev1.PersistentVolume{}), handler.EnqueueRequestsFromMapFunc(
+		func(ctx context.Context, obj client.Object) (reqs []reconcile.Request) {
 			pv := obj.(*corev1.PersistentVolume)
 			dvList := &cdiv1.DataVolumeList{}
-			if err := mgr.GetClient().List(context.TODO(), dvList, client.MatchingFields{dvPhaseField: ""}); err != nil {
+			if err := mgr.GetClient().List(ctx, dvList, client.MatchingFields{dvPhaseField: ""}); err != nil {
 				return
 			}
 			for _, dv := range dvList.Items {
@@ -324,7 +343,7 @@ func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeControl
 					storage.StorageClassName != nil &&
 					*storage.StorageClassName == pv.Spec.StorageClassName &&
 					pv.Status.Phase == corev1.VolumeAvailable &&
-					getDataVolumeOp(mgr.GetLogger(), &dv, mgr.GetClient()) == op {
+					getDataVolumeOp(ctx, mgr.GetLogger(), &dv, mgr.GetClient()) == op {
 					reqs = append(reqs, reconcile.Request{NamespacedName: types.NamespacedName{Name: dv.Name, Namespace: dv.Namespace}})
 				}
 			}
@@ -338,11 +357,11 @@ func addDataVolumeControllerCommonWatches(mgr manager.Manager, dataVolumeControl
 	return nil
 }
 
-func getDataVolumeOp(log logr.Logger, dv *cdiv1.DataVolume, client client.Client) dataVolumeOp {
+func getDataVolumeOp(ctx context.Context, log logr.Logger, dv *cdiv1.DataVolume, client client.Client) dataVolumeOp {
 	src := dv.Spec.Source
 
 	if dv.Spec.SourceRef != nil {
-		return getSourceRefOp(log, dv, client)
+		return getSourceRefOp(ctx, log, dv, client)
 	}
 	if src != nil && src.PVC != nil {
 		return dataVolumePvcClone
@@ -366,14 +385,14 @@ func getDataVolumeOp(log logr.Logger, dv *cdiv1.DataVolume, client client.Client
 	return dataVolumeNop
 }
 
-func getSourceRefOp(log logr.Logger, dv *cdiv1.DataVolume, client client.Client) dataVolumeOp {
+func getSourceRefOp(ctx context.Context, log logr.Logger, dv *cdiv1.DataVolume, client client.Client) dataVolumeOp {
 	dataSource := &cdiv1.DataSource{}
 	ns := dv.Namespace
 	if dv.Spec.SourceRef.Namespace != nil && *dv.Spec.SourceRef.Namespace != "" {
 		ns = *dv.Spec.SourceRef.Namespace
 	}
 	nn := types.NamespacedName{Namespace: ns, Name: dv.Spec.SourceRef.Name}
-	if err := client.Get(context.TODO(), nn, dataSource); err != nil {
+	if err := client.Get(ctx, nn, dataSource); err != nil {
 		log.Error(err, "Unable to get DataSource", "namespacedName", nn)
 		return dataVolumeNop
 	}
@@ -388,6 +407,26 @@ func getSourceRefOp(log logr.Logger, dv *cdiv1.DataVolume, client client.Client)
 	}
 }
 
+func updatePendingDataVolumesGauge(ctx context.Context, log logr.Logger, dv *cdiv1.DataVolume, c client.Client) {
+	if cc.GetStorageClassFromDVSpec(dv) != nil {
+		return
+	}
+
+	dvList := &cdiv1.DataVolumeList{}
+	if err := c.List(ctx, dvList, client.MatchingFields{dvPhaseField: string(cdiv1.Pending)}); err != nil {
+		log.V(3).Error(err, "Failed listing the pending DataVolumes")
+		return
+	}
+
+	dvCount := 0
+	for _, dv := range dvList.Items {
+		if cc.GetStorageClassFromDVSpec(&dv) == nil {
+			dvCount++
+		}
+	}
+	metrics.SetDataVolumePending(dvCount)
+}
+
 type dvController interface {
 	reconcile.Reconciler
 	sync(log logr.Logger, req reconcile.Request) (dvSyncResult, error)
@@ -472,6 +511,10 @@ func (r *ReconcilerBase) syncDvPvcState(log logr.Logger, req reconcile.Request,
 		return syncState, err
 	}
 
+	if err := r.handleDelayedAnnotations(&syncState, log); err != nil || syncState.result != nil {
+		return syncState, err
+	}
+
 	if err = updateDataVolumeDefaultInstancetypeLabels(r.client, &syncState); err != nil {
 		return syncState, err
 	}
@@ -589,6 +632,37 @@ func (r *ReconcilerBase) handleStaticVolume(syncState *dvSyncState, log logr.Log
 	return fmt.Errorf("DataVolume bound to unexpected PV %s", syncState.pvc.Spec.VolumeName)
 }
 
+func (r *ReconcilerBase) handleDelayedAnnotations(syncState *dvSyncState, log logr.Logger) error {
+	dataVolume := syncState.dv
+	if dataVolume.Status.Phase != cdiv1.Succeeded {
+		return nil
+	}
+
+	if syncState.pvc == nil {
+		return nil
+	}
+
+	pvcCpy := syncState.pvc.DeepCopy()
+	for _, anno := range delayedAnnotations {
+		if val, ok := dataVolume.Annotations[anno]; ok {
+			// only add if not already present
+			if _, ok := pvcCpy.Annotations[anno]; !ok {
+				cc.AddAnnotation(pvcCpy, anno, val)
+			}
+		}
+	}
+
+	if !reflect.DeepEqual(syncState.pvc, pvcCpy) {
+		if err := r.updatePVC(pvcCpy); err != nil {
+			return err
+		}
+		syncState.pvc = pvcCpy
+		syncState.result = &reconcile.Result{}
+	}
+
+	return nil
+}
+
 func (r *ReconcilerBase) getAvailableVolumesForDV(syncState *dvSyncState, log logr.Logger) ([]string, error) {
 	pvList := &corev1.PersistentVolumeList{}
 	fields := client.MatchingFields{claimRefField: claimRefIndexKeyFunc(syncState.dv.Namespace, syncState.dv.Name)}
@@ -604,7 +678,7 @@ func (r *ReconcilerBase) getAvailableVolumesForDV(syncState *dvSyncState, log lo
 			pvc := &corev1.PersistentVolumeClaim{
 				Spec: *syncState.pvcSpec,
 			}
-			if err := checkVolumeSatisfyClaim(&pv, pvc); err != nil {
+			if err := CheckVolumeSatisfyClaim(&pv, pvc); err != nil {
 				continue
 			}
 			log.Info("Found matching volume for DV", "pv", pv.Name)
@@ -615,7 +689,7 @@ func (r *ReconcilerBase) getAvailableVolumesForDV(syncState *dvSyncState, log lo
 }
 
 func (r *ReconcilerBase) handlePrePopulation(dv *cdiv1.DataVolume, pvc *corev1.PersistentVolumeClaim) {
-	if pvc.Status.Phase == corev1.ClaimBound && pvcIsPopulated(pvc, dv) {
+	if pvc.Status.Phase == corev1.ClaimBound && pvcIsPopulatedForDataVolume(pvc, dv) {
 		cc.AddAnnotation(dv, cc.AnnPrePopulated, pvc.Name)
 	}
 }
@@ -631,7 +705,11 @@ func (r *ReconcilerBase) validatePVC(dv *cdiv1.DataVolume, pvc *corev1.Persisten
 	// If the PVC is not controlled by this DataVolume resource, we should log
 	// a warning to the event recorder and return
 	if !metav1.IsControlledBy(pvc, dv) {
-		if pvcIsPopulated(pvc, dv) {
+		requiresWork, err := r.pvcRequiresWork(pvc, dv)
+		if err != nil {
+			return err
+		}
+		if !requiresWork {
 			if err := r.addOwnerRef(pvc, dv); err != nil {
 				return err
 			}
@@ -822,15 +900,23 @@ func (r *ReconcilerBase) updateStatus(req reconcile.Request, phaseSync *statusPh
 		dataVolumeCopy.Status.ClaimName = pvc.Name
 
 		phase := pvc.Annotations[cc.AnnPodPhase]
-		if phase == string(cdiv1.Succeeded) {
+		requiresWork, err := r.pvcRequiresWork(pvc, dataVolumeCopy)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+		if phase == string(cdiv1.Succeeded) && requiresWork {
 			if err := dvc.updateStatusPhase(pvc, dataVolumeCopy, &event); err != nil {
 				return reconcile.Result{}, err
 			}
 		} else {
 			switch pvc.Status.Phase {
 			case corev1.ClaimPending:
-				if err := r.updateStatusPVCPending(pvc, dvc, dataVolumeCopy, &event); err != nil {
-					return reconcile.Result{}, err
+				if requiresWork {
+					if err := r.updateStatusPVCPending(pvc, dvc, dataVolumeCopy, &event); err != nil {
+						return reconcile.Result{}, err
+					}
+				} else {
+					dataVolumeCopy.Status.Phase = cdiv1.Succeeded
 				}
 			case corev1.ClaimBound:
 				switch dataVolumeCopy.Status.Phase {
@@ -842,12 +928,12 @@ func (r *ReconcilerBase) updateStatus(req reconcile.Request, phaseSync *statusPh
 					dataVolumeCopy.Status.Phase = cdiv1.PVCBound
 				}
 
-				if pvcIsPopulated(pvc, dataVolumeCopy) {
-					dataVolumeCopy.Status.Phase = cdiv1.Succeeded
-				} else {
+				if requiresWork {
 					if err := dvc.updateStatusPhase(pvc, dataVolumeCopy, &event); err != nil {
 						return reconcile.Result{}, err
 					}
+				} else {
+					dataVolumeCopy.Status.Phase = cdiv1.Succeeded
 				}
 
 			case corev1.ClaimLost:
@@ -1042,6 +1128,21 @@ func (r *ReconcilerBase) newPersistentVolumeClaim(dataVolume *cdiv1.DataVolume,
 		annotations[cc.AnnPriorityClassName] = dataVolume.Spec.PriorityClassName
 	}
 	annotations[cc.AnnPreallocationRequested] = strconv.FormatBool(cc.GetPreallocation(context.TODO(), r.client, dataVolume.Spec.Preallocation))
+	annotations[cc.AnnCreatedForDataVolume] = string(dataVolume.UID)
+
+	if dataVolume.Spec.Storage != nil && labels[common.PvcApplyStorageProfileLabel] == "true" {
+		isWebhookPvcRenderingEnabled, err := featuregates.IsWebhookPvcRenderingEnabled(r.client)
+		if err != nil {
+			return nil, err
+		}
+		if isWebhookPvcRenderingEnabled {
+			labels[common.PvcApplyStorageProfileLabel] = "true"
+			if targetPvcSpec.VolumeMode == nil {
+				targetPvcSpec.VolumeMode = ptr.To[corev1.PersistentVolumeMode](cdiv1.PersistentVolumeFromStorageProfile)
+			}
+		}
+	}
+
 	pvc := &corev1.PersistentVolumeClaim{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace:   namespace,
@@ -1073,6 +1174,10 @@ func (r *ReconcilerBase) newPersistentVolumeClaim(dataVolume *cdiv1.DataVolume,
 		pvc.Annotations[cc.AnnOwnerUID] = string(dataVolume.UID)
 	}
 
+	for _, anno := range delayedAnnotations {
+		delete(pvc.Annotations, anno)
+	}
+
 	return pvc, nil
 }
 
@@ -1129,6 +1234,14 @@ func (r *ReconcilerBase) shouldBeMarkedWaitForFirstConsumer(pvc *corev1.Persiste
 	return res, nil
 }
 
+func (r *ReconcilerBase) shouldReconcileVolumeSourceCR(syncState *dvSyncState) bool {
+	if syncState.pvc == nil {
+		return true
+	}
+	phase := syncState.pvc.Annotations[cc.AnnPodPhase]
+	return phase != string(corev1.PodSucceeded) || syncState.dvMutated.Status.Phase != cdiv1.Succeeded
+}
+
 // shouldBeMarkedPendingPopulation decides whether we should mark DV as PendingPopulation
 func (r *ReconcilerBase) shouldBeMarkedPendingPopulation(pvc *corev1.PersistentVolumeClaim) (bool, error) {
 	wffc, err := r.storageClassWaitForFirstConsumer(pvc.Spec.StorageClassName)
@@ -1192,3 +1305,20 @@ func (r *ReconcilerBase) shouldUseCDIPopulator(syncState *dvSyncState) (bool, er
 
 	return usePopulator, nil
 }
+
+func (r *ReconcilerBase) pvcRequiresWork(pvc *corev1.PersistentVolumeClaim, dv *cdiv1.DataVolume) (bool, error) {
+	if pvc == nil || dv == nil {
+		return true, nil
+	}
+	if pvcIsPopulatedForDataVolume(pvc, dv) {
+		return false, nil
+	}
+	canAdopt, err := cc.AllowClaimAdoption(r.client, pvc, dv)
+	if err != nil {
+		return true, err
+	}
+	if canAdopt {
+		return false, nil
+	}
+	return true, nil
+}
