@@ -125,7 +125,7 @@ func (r *PvcCloneReconciler) addDataVolumeCloneControllerWatches(mgr manager.Man
 		return err
 	}
 
-	if err := r.addVolumeCloneSourceWatch(datavolumeController); err != nil {
+	if err := r.addVolumeCloneSourceWatch(mgr, datavolumeController); err != nil {
 		return err
 	}
 
@@ -152,10 +152,10 @@ func addDataSourceWatch(mgr manager.Manager, c controller.Controller) error {
 		return err
 	}
 
-	mapToDataVolume := func(obj client.Object) (reqs []reconcile.Request) {
+	mapToDataVolume := func(ctx context.Context, obj client.Object) (reqs []reconcile.Request) {
 		var dvs cdiv1.DataVolumeList
 		matchingFields := client.MatchingFields{dvDataSourceField: getKey(obj.GetNamespace(), obj.GetName())}
-		if err := mgr.GetClient().List(context.TODO(), &dvs, matchingFields); err != nil {
+		if err := mgr.GetClient().List(ctx, &dvs, matchingFields); err != nil {
 			c.GetLogger().Error(err, "Unable to list DataVolumes", "matchingFields", matchingFields)
 			return
 		}
@@ -165,7 +165,7 @@ func addDataSourceWatch(mgr manager.Manager, c controller.Controller) error {
 		return
 	}
 
-	if err := c.Watch(&source.Kind{Type: &cdiv1.DataSource{}},
+	if err := c.Watch(source.Kind(mgr.GetCache(), &cdiv1.DataSource{}),
 		handler.EnqueueRequestsFromMapFunc(mapToDataVolume),
 	); err != nil {
 		return err
@@ -252,11 +252,14 @@ func (r *PvcCloneReconciler) syncClone(log logr.Logger, req reconcile.Request) (
 	pvc := syncRes.pvc
 	pvcSpec := syncRes.pvcSpec
 	datavolume := syncRes.dvMutated
-	pvcPopulated := pvcIsPopulated(pvc, datavolume)
 	staticProvisionPending := checkStaticProvisionPending(pvc, datavolume)
 	prePopulated := dvIsPrePopulated(datavolume)
+	requiresWork, err := r.pvcRequiresWork(pvc, datavolume)
+	if err != nil {
+		return syncRes, err
+	}
 
-	if pvcPopulated || prePopulated || staticProvisionPending {
+	if !requiresWork || prePopulated || staticProvisionPending {
 		return syncRes, nil
 	}
 
@@ -517,7 +520,11 @@ func (r *PvcCloneReconciler) detectCloneSize(syncState *dvSyncState) (bool, erro
 		return false, err
 	}
 
+	if syncState.pvcSpec.Resources.Requests == nil {
+		syncState.pvcSpec.Resources.Requests = corev1.ResourceList{}
+	}
 	syncState.pvcSpec.Resources.Requests[corev1.ResourceStorage] = targetCapacity
+
 	return true, nil
 }
 
