@@ -32,11 +32,10 @@ import (
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
@@ -153,6 +152,9 @@ var _ = Describe("All DataVolume Tests", func() {
 				Expect(err).ToNot(HaveOccurred())
 				Expect(vcs.Spec.Source.Kind).To(Equal("PersistentVolumeClaim"))
 				Expect(vcs.Spec.Source.Name).To(Equal(srcPvc.Name))
+				val, ok := pvc.Annotations[AnnCreatedForDataVolume]
+				Expect(val).To(Equal(string(dv.UID)))
+				Expect(ok).To(BeTrue())
 			},
 				Entry("with same namespace", metav1.NamespaceDefault),
 				Entry("with different namespace", "source-ns"),
@@ -172,7 +174,7 @@ var _ = Describe("All DataVolume Tests", func() {
 				}
 				pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 					Kind:       "DataVolume",
-					Controller: pointer.Bool(true),
+					Controller: ptr.To[bool](true),
 					Name:       "test-dv",
 					UID:        dv.UID,
 				})
@@ -210,7 +212,7 @@ var _ = Describe("All DataVolume Tests", func() {
 				pvc := CreatePvcInStorageClass("test-dv", metav1.NamespaceDefault, &scName, anno, nil, corev1.ClaimPending)
 				pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 					Kind:       "DataVolume",
-					Controller: pointer.Bool(true),
+					Controller: ptr.To[bool](true),
 					Name:       "test-dv",
 					UID:        dv.UID,
 				})
@@ -252,7 +254,7 @@ var _ = Describe("All DataVolume Tests", func() {
 				}
 				pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 					Kind:       "DataVolume",
-					Controller: pointer.Bool(true),
+					Controller: ptr.To[bool](true),
 					Name:       "test-dv",
 					UID:        dv.UID,
 				})
@@ -313,7 +315,7 @@ var _ = Describe("All DataVolume Tests", func() {
 				}
 				pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 					Kind:       "DataVolume",
-					Controller: pointer.Bool(true),
+					Controller: ptr.To[bool](true),
 					Name:       "test-dv",
 					UID:        dv.UID,
 				})
@@ -342,7 +344,7 @@ var _ = Describe("All DataVolume Tests", func() {
 	})
 
 	var _ = Describe("Reconcile Datavolume status", func() {
-		DescribeTable("DV phase", func(testDv runtime.Object, current, expected cdiv1.DataVolumePhase, pvcPhase corev1.PersistentVolumeClaimPhase, podPhase corev1.PodPhase, ann, expectedEvent string, extraAnnotations ...string) {
+		DescribeTable("DV phase", func(testDv client.Object, current, expected cdiv1.DataVolumePhase, pvcPhase corev1.PersistentVolumeClaimPhase, podPhase corev1.PodPhase, ann, expectedEvent string, extraAnnotations ...string) {
 			scName := "testpvc"
 			srcPvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
 			sc := CreateStorageClassWithProvisioner(scName, map[string]string{AnnDefaultStorageClass: "true"}, map[string]string{}, "csi-plugin")
@@ -459,28 +461,114 @@ var _ = Describe("All DataVolume Tests", func() {
 		It("Validate clone already populated without source completes", func() {
 			dv := newCloneDataVolume("test-dv")
 			storageProfile := createStorageProfile(scName, nil, FilesystemMode)
-			pvcAnnotations := make(map[string]string)
-			pvcAnnotations[AnnPopulatedFor] = "test-dv"
 			pvc := CreatePvcInStorageClass("test-dv", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
 			pvc.SetAnnotations(make(map[string]string))
 			pvc.GetAnnotations()[AnnPopulatedFor] = "test-dv"
 			reconciler = createCloneReconciler(dv, pvc, storageProfile, sc)
 
-			syncState := dvSyncState{dv: dv, dvMutated: dv.DeepCopy(), pvc: pvc}
-			reconciler.handlePrePopulation(syncState.dvMutated, syncState.pvc)
-			err := reconciler.client.Update(context.TODO(), syncState.dvMutated)
+			result, err := reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}})
 			Expect(err).ToNot(HaveOccurred())
+			Expect(result.Requeue).To(BeFalse())
+			Expect(result.RequeueAfter).To(BeZero())
 
-			result, err := reconciler.updateStatus(getReconcileRequest(dv), syncState.phaseSync, reconciler)
-
-			Expect(err).ToNot(HaveOccurred())
 			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, dv)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(dv.Status.ClaimName).To(Equal("test-dv"))
 			Expect(dv.Status.Phase).To(Equal(cdiv1.Succeeded))
 			Expect(dv.Annotations[AnnPrePopulated]).To(Equal("test-dv"))
 			Expect(dv.Annotations[AnnCloneType]).To(BeEmpty())
-			Expect(result).To(Equal(reconcile.Result{}))
+
+			pvc = &corev1.PersistentVolumeClaim{}
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, pvc)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(pvc.OwnerReferences).To(HaveLen(1))
+			Expect(pvc.OwnerReferences[0].Name).To(Equal("test-dv"))
+			Expect(pvc.OwnerReferences[0].Kind).To(Equal("DataVolume"))
+		})
+
+		It("Validate clone will adopt PVC (with annotation)", func() {
+			dv := newCloneDataVolume("test-dv")
+			AddAnnotation(dv, AnnAllowClaimAdoption, "true")
+			storageProfile := createStorageProfile(scName, nil, FilesystemMode)
+			pvc := CreatePvcInStorageClass("test-dv", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
+			reconciler = createCloneReconciler(dv, pvc, storageProfile, sc)
+
+			result, err := reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(result.Requeue).To(BeFalse())
+			Expect(result.RequeueAfter).To(BeZero())
+
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, dv)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(dv.Status.ClaimName).To(Equal("test-dv"))
+			Expect(dv.Status.Phase).To(Equal(cdiv1.Succeeded))
+			Expect(dv.Annotations[AnnCloneType]).To(BeEmpty())
+
+			pvc = &corev1.PersistentVolumeClaim{}
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, pvc)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(pvc.OwnerReferences).To(HaveLen(1))
+			Expect(pvc.OwnerReferences[0].Name).To(Equal("test-dv"))
+			Expect(pvc.OwnerReferences[0].Kind).To(Equal("DataVolume"))
+			_, ok := pvc.Annotations[AnnCreatedForDataVolume]
+			Expect(ok).To(BeFalse())
+		})
+
+		It("Validate clone will adopt unbound PVC (with annotation)", func() {
+			dv := newCloneDataVolume("test-dv")
+			AddAnnotation(dv, AnnAllowClaimAdoption, "true")
+			storageProfile := createStorageProfile(scName, nil, FilesystemMode)
+			pvc := CreatePvcInStorageClass("test-dv", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimPending)
+			pvc.Spec.VolumeName = ""
+			reconciler = createCloneReconciler(dv, pvc, storageProfile, sc)
+
+			result, err := reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(result.Requeue).To(BeFalse())
+			Expect(result.RequeueAfter).To(BeZero())
+
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, dv)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(dv.Status.ClaimName).To(Equal("test-dv"))
+			Expect(dv.Status.Phase).To(Equal(cdiv1.Succeeded))
+			Expect(dv.Annotations[AnnCloneType]).To(BeEmpty())
+
+			pvc = &corev1.PersistentVolumeClaim{}
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, pvc)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(pvc.OwnerReferences).To(HaveLen(1))
+			Expect(pvc.OwnerReferences[0].Name).To(Equal("test-dv"))
+			Expect(pvc.OwnerReferences[0].Kind).To(Equal("DataVolume"))
+			_, ok := pvc.Annotations[AnnCreatedForDataVolume]
+			Expect(ok).To(BeFalse())
+		})
+
+		It("Validate clone will adopt PVC (with featuregate)", func() {
+			dv := newCloneDataVolume("test-dv")
+			storageProfile := createStorageProfile(scName, nil, FilesystemMode)
+			pvc := CreatePvcInStorageClass("test-dv", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
+			featureGates := []string{featuregates.DataVolumeClaimAdoption, featuregates.HonorWaitForFirstConsumer}
+			reconciler = createCloneReconcilerWithFeatureGates(featureGates, dv, pvc, storageProfile, sc)
+
+			result, err := reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(result.Requeue).To(BeFalse())
+			Expect(result.RequeueAfter).To(BeZero())
+
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, dv)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(dv.Status.ClaimName).To(Equal("test-dv"))
+			Expect(dv.Status.Phase).To(Equal(cdiv1.Succeeded))
+			Expect(dv.Annotations[AnnCloneType]).To(BeEmpty())
+
+			pvc = &corev1.PersistentVolumeClaim{}
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}, pvc)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(pvc.OwnerReferences).To(HaveLen(1))
+			Expect(pvc.OwnerReferences[0].Name).To(Equal("test-dv"))
+			Expect(pvc.OwnerReferences[0].Kind).To(Equal("DataVolume"))
+			_, ok := pvc.Annotations[AnnCreatedForDataVolume]
+			Expect(ok).To(BeFalse())
 		})
 
 		DescribeTable("Validation mechanism rejects or accepts the clone depending on the contentType combination",
@@ -523,6 +611,7 @@ var _ = Describe("All DataVolume Tests", func() {
 	var _ = Describe("Clone with empty storage size", func() {
 		scName := "testsc"
 		accessMode := []corev1.PersistentVolumeAccessMode{corev1.ReadOnlyMany}
+		annKubevirt := map[string]string{AnnContentType: "kubevirt"}
 		sc := CreateStorageClassWithProvisioner(scName, map[string]string{
 			AnnDefaultStorageClass: "true",
 		}, map[string]string{}, "csi-plugin")
@@ -531,18 +620,27 @@ var _ = Describe("All DataVolume Tests", func() {
 			return &dvSyncState{dv: dv, dvMutated: dv.DeepCopy(), pvc: pvc, pvcSpec: pvcSpec}
 		}
 
+		createTargetPvc := func(pvcSpec *corev1.PersistentVolumeClaimSpec) *corev1.PersistentVolumeClaim {
+			return &corev1.PersistentVolumeClaim{
+				ObjectMeta: metav1.ObjectMeta{
+					Annotations: map[string]string{AnnCloneType: string(cdiv1.CloneStrategyHostAssisted)},
+				},
+				Spec: *pvcSpec,
+			}
+		}
 		// detectCloneSize tests
 		It("Size-detection fails when source PVC is not attainable", func() {
 			dv := newCloneDataVolumeWithEmptyStorage("test-dv", "default")
 			cloneStrategy := cdiv1.CloneStrategyHostAssisted
-			targetPvc := &corev1.PersistentVolumeClaim{}
 			storageProfile := createStorageProfileWithCloneStrategy(scName, []cdiv1.ClaimPropertySet{
 				{AccessModes: accessMode, VolumeMode: &BlockMode}}, &cloneStrategy)
 
 			reconciler := createCloneReconciler(dv, storageProfile, sc)
 			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 			Expect(err).ToNot(HaveOccurred())
-			done, err := reconciler.detectCloneSize(syncState(dv, targetPvc, pvcSpec))
+
+			targetPvc := createTargetPvc(pvcSpec)
+			done, err := reconciler.detectCloneSize(syncState(dv, targetPvc, &targetPvc.Spec))
 			Expect(err).To(HaveOccurred())
 			Expect(done).To(BeFalse())
 			Expect(k8serrors.IsNotFound(err)).To(BeTrue())
@@ -563,20 +661,21 @@ var _ = Describe("All DataVolume Tests", func() {
 					Phase: cdiv1.ImportInProgress,
 				},
 			}
-			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
+			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, annKubevirt, nil, corev1.ClaimBound)
 			pvc.OwnerReferences = []metav1.OwnerReference{
 				{
 					Kind:       "DataVolume",
 					Name:       sourceDV.Name,
-					Controller: pointer.Bool(true),
+					Controller: ptr.To[bool](true),
 				},
 			}
-			AddAnnotation(pvc, AnnContentType, "kubevirt")
 			reconciler := createCloneReconciler(dv, sourceDV, pvc, storageProfile, sc)
 
-			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, pvc)
+			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 			Expect(err).ToNot(HaveOccurred())
-			done, err := reconciler.detectCloneSize(syncState(dv, pvc, pvcSpec))
+
+			targetPvc := createTargetPvc(pvcSpec)
+			done, err := reconciler.detectCloneSize(syncState(dv, targetPvc, &targetPvc.Spec))
 			Expect(err).ToNot(HaveOccurred())
 			Expect(done).To(BeFalse())
 			By("Checking events recorded")
@@ -597,14 +696,14 @@ var _ = Describe("All DataVolume Tests", func() {
 			storageProfile := createStorageProfileWithCloneStrategy(scName, []cdiv1.ClaimPropertySet{
 				{AccessModes: accessMode, VolumeMode: &BlockMode}}, &cloneStrategy)
 
-			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
-			pvc.SetAnnotations(make(map[string]string))
-			pvc.Annotations[AnnContentType] = "kubevirt"
+			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, annKubevirt, nil, corev1.ClaimBound)
 			reconciler := createCloneReconciler(dv, pvc, storageProfile, sc)
 
-			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, pvc)
+			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 			Expect(err).ToNot(HaveOccurred())
-			done, err := reconciler.detectCloneSize(syncState(dv, pvc, pvcSpec))
+
+			targetPvc := createTargetPvc(pvcSpec)
+			done, err := reconciler.detectCloneSize(syncState(dv, targetPvc, &targetPvc.Spec))
 			Expect(err).ToNot(HaveOccurred())
 			Expect(done).To(BeFalse())
 			By("Checking events recorded")
@@ -626,9 +725,7 @@ var _ = Describe("All DataVolume Tests", func() {
 			storageProfile := createStorageProfileWithCloneStrategy(scName, []cdiv1.ClaimPropertySet{
 				{AccessModes: accessMode, VolumeMode: &BlockMode}}, &cloneStrategy)
 
-			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
-			pvc.SetAnnotations(make(map[string]string))
-			pvc.Annotations[AnnContentType] = "kubevirt"
+			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, annKubevirt, nil, corev1.ClaimBound)
 			reconciler := createCloneReconciler(dv, pvc, storageProfile, sc)
 
 			// Prepare the size-detection Pod with the required information
@@ -638,9 +735,11 @@ var _ = Describe("All DataVolume Tests", func() {
 			Expect(err).ToNot(HaveOccurred())
 
 			// Checks
-			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, pvc)
+			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 			Expect(err).ToNot(HaveOccurred())
-			done, err := reconciler.detectCloneSize(syncState(dv, pvc, pvcSpec))
+
+			targetPvc := createTargetPvc(pvcSpec)
+			done, err := reconciler.detectCloneSize(syncState(dv, targetPvc, &targetPvc.Spec))
 			Expect(err).To(HaveOccurred())
 			Expect(err).To(Equal(ErrInvalidTermMsg))
 			Expect(done).To(BeFalse())
@@ -656,10 +755,7 @@ var _ = Describe("All DataVolume Tests", func() {
 			cloneStrategy := cdiv1.CloneStrategyHostAssisted
 			storageProfile := createStorageProfileWithCloneStrategy(scName, []cdiv1.ClaimPropertySet{
 				{AccessModes: accessMode, VolumeMode: &BlockMode}}, &cloneStrategy)
-
-			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
-			pvc.SetAnnotations(make(map[string]string))
-			pvc.Annotations[AnnContentType] = "kubevirt"
+			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, annKubevirt, nil, corev1.ClaimBound)
 			reconciler := createCloneReconciler(dv, pvc, storageProfile, sc)
 
 			// Prepare the size-detection Pod with the required information
@@ -679,19 +775,22 @@ var _ = Describe("All DataVolume Tests", func() {
 			Expect(err).ToNot(HaveOccurred())
 
 			// Get the expected value
-			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, pvc)
+			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 			Expect(err).ToNot(HaveOccurred())
-			expectedSize, err := InflateSizeWithOverhead(context.TODO(), reconciler.client, int64(100), pvcSpec)
+
+			targetPvc := createTargetPvc(pvcSpec)
+			expectedSize, err := InflateSizeWithOverhead(context.TODO(), reconciler.client, int64(100), &targetPvc.Spec)
 			Expect(err).ToNot(HaveOccurred())
 			expectedSizeInt64, _ := expectedSize.AsInt64()
 
 			// Checks
-			syncState := syncState(dv, pvc, pvcSpec)
+			syncState := syncState(dv, targetPvc, &targetPvc.Spec)
 			done, err := reconciler.detectCloneSize(syncState)
+
 			Expect(err).ToNot(HaveOccurred())
 			Expect(done).To(BeTrue())
 			Expect(syncState.dvMutated.Annotations[AnnPermissiveClone]).To(Equal("true"))
-			targetSize := pvcSpec.Resources.Requests[corev1.ResourceStorage]
+			targetSize := targetPvc.Spec.Resources.Requests.Storage()
 			targetSizeInt64, _ := targetSize.AsInt64()
 			Expect(targetSizeInt64).To(Equal(expectedSizeInt64))
 		})
@@ -703,27 +802,27 @@ var _ = Describe("All DataVolume Tests", func() {
 				{AccessModes: accessMode, VolumeMode: &BlockMode}}, &cloneStrategy)
 
 			// Prepare the source PVC with the required annotations
-			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, nil, nil, corev1.ClaimBound)
-			pvc.SetAnnotations(make(map[string]string))
-			pvc.GetAnnotations()[AnnVirtualImageSize] = "100" // Mock value
-			pvc.GetAnnotations()[AnnSourceCapacity] = string(pvc.Status.Capacity.Storage().String())
-			pvc.GetAnnotations()[AnnContentType] = "kubevirt"
+			pvc := CreatePvcInStorageClass("test", metav1.NamespaceDefault, &scName, annKubevirt, nil, corev1.ClaimBound)
+			pvc.Annotations[AnnVirtualImageSize] = "100" // Mock value
+			pvc.Annotations[AnnSourceCapacity] = string(pvc.Status.Capacity.Storage().String())
 			reconciler := createCloneReconciler(dv, pvc, storageProfile, sc)
 
 			// Get the expected value
-			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, pvc)
+			pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 			Expect(err).ToNot(HaveOccurred())
-			expectedSize, err := InflateSizeWithOverhead(context.TODO(), reconciler.client, int64(100), pvcSpec)
+
+			targetPvc := createTargetPvc(pvcSpec)
+			expectedSize, err := InflateSizeWithOverhead(context.TODO(), reconciler.client, int64(100), &targetPvc.Spec)
 			Expect(err).ToNot(HaveOccurred())
 			expectedSizeInt64, _ := expectedSize.AsInt64()
 
 			// Checks
-			syncState := syncState(dv, pvc, pvcSpec)
+			syncState := syncState(dv, targetPvc, &targetPvc.Spec)
 			done, err := reconciler.detectCloneSize(syncState)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(done).To(BeTrue())
 			Expect(syncState.dvMutated.Annotations[AnnPermissiveClone]).To(Equal("true"))
-			targetSize := pvcSpec.Resources.Requests[corev1.ResourceStorage]
+			targetSize := targetPvc.Spec.Resources.Requests.Storage()
 			targetSizeInt64, _ := targetSize.AsInt64()
 			Expect(targetSizeInt64).To(Equal(expectedSizeInt64))
 		})
@@ -738,49 +837,51 @@ var _ = Describe("All DataVolume Tests", func() {
 				pvc.Spec.VolumeMode = &volumeMode
 				reconciler := createCloneReconciler(dv, pvc, storageProfile, sc)
 
-				pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, pvc)
+				pvcSpec, err := renderPvcSpec(reconciler.client, reconciler.recorder, reconciler.log, dv, nil)
 				Expect(err).ToNot(HaveOccurred())
+
+				targetPvc := createTargetPvc(pvcSpec)
 				expectedSize := *pvc.Status.Capacity.Storage()
-				done, err := reconciler.detectCloneSize(syncState(dv, pvc, pvcSpec))
+				expectedSizeInt64, _ := expectedSize.AsInt64()
+
+				syncState := syncState(dv, targetPvc, &targetPvc.Spec)
+				done, err := reconciler.detectCloneSize(syncState)
 				Expect(err).ToNot(HaveOccurred())
 				Expect(done).To(BeTrue())
-				Expect(pvc.Spec.Resources.Requests.Storage().Cmp(expectedSize)).To(Equal(0))
+
+				targetSize := targetPvc.Spec.Resources.Requests.Storage()
+				targetSizeInt64, _ := targetSize.AsInt64()
+				Expect(targetSizeInt64).To(Equal(expectedSizeInt64))
 			},
 			Entry("hostAssited with empty size and 'Block' volume mode", cdiv1.CloneStrategyHostAssisted, BlockMode),
 		)
 	})
 })
 
-func createCloneReconcilerWFFCDisabled(objects ...runtime.Object) *PvcCloneReconciler {
+func createCloneReconcilerWFFCDisabled(objects ...client.Object) *PvcCloneReconciler {
+	return createCloneReconcilerWithFeatureGates(nil, objects...)
+}
+
+func createCloneReconciler(objects ...client.Object) *PvcCloneReconciler {
+	return createCloneReconcilerWithFeatureGates([]string{featuregates.HonorWaitForFirstConsumer}, objects...)
+}
+
+func createCloneReconcilerWithFeatureGates(featireGates []string, objects ...client.Object) *PvcCloneReconciler {
 	cdiConfig := MakeEmptyCDIConfigSpec(common.ConfigName)
 	cdiConfig.Status = cdiv1.CDIConfigStatus{
 		ScratchSpaceStorageClass: testStorageClass,
 	}
-	cdiConfig.Spec.FeatureGates = []string{}
+	cdiConfig.Spec.FeatureGates = featireGates
 
-	objs := []runtime.Object{}
+	objs := []client.Object{}
 	objs = append(objs, objects...)
 	objs = append(objs, cdiConfig)
 
 	return createCloneReconcilerWithoutConfig(objs...)
 }
 
-func createCloneReconciler(objects ...runtime.Object) *PvcCloneReconciler {
-	cdiConfig := MakeEmptyCDIConfigSpec(common.ConfigName)
-	cdiConfig.Status = cdiv1.CDIConfigStatus{
-		ScratchSpaceStorageClass: testStorageClass,
-	}
-	cdiConfig.Spec.FeatureGates = []string{featuregates.HonorWaitForFirstConsumer}
-
-	objs := []runtime.Object{}
-	objs = append(objs, objects...)
-	objs = append(objs, cdiConfig)
-
-	return createCloneReconcilerWithoutConfig(objs...)
-}
-
-func createCloneReconcilerWithoutConfig(objects ...runtime.Object) *PvcCloneReconciler {
-	objs := []runtime.Object{}
+func createCloneReconcilerWithoutConfig(objects ...client.Object) *PvcCloneReconciler {
+	objs := []client.Object{}
 	objs = append(objs, objects...)
 
 	// Register operator types with the runtime scheme.
@@ -793,7 +894,8 @@ func createCloneReconcilerWithoutConfig(objects ...runtime.Object) *PvcCloneReco
 
 	builder := fake.NewClientBuilder().
 		WithScheme(s).
-		WithRuntimeObjects(objs...)
+		WithObjects(objs...).
+		WithStatusSubresource(objs...)
 
 	for _, ia := range getIndexArgs() {
 		builder = builder.WithIndex(ia.obj, ia.field, ia.extractValue)
