@@ -32,7 +32,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/cache"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
@@ -86,7 +86,7 @@ func NewSnapshotCloneController(
 			clonerImage:         clonerImage,
 			importerImage:       importerImage,
 			pullPolicy:          pullPolicy,
-			cloneSourceAPIGroup: pointer.String(snapshotv1.GroupName),
+			cloneSourceAPIGroup: ptr.To[string](snapshotv1.GroupName),
 			cloneSourceKind:     "VolumeSnapshot",
 			shortTokenValidator: cc.NewCloneTokenValidator(common.CloneTokenIssuer, tokenPublicKey),
 			longTokenValidator:  cc.NewCloneTokenValidator(common.ExtendedCloneTokenIssuer, tokenPublicKey),
@@ -130,7 +130,7 @@ func (r *SnapshotCloneReconciler) addDataVolumeSnapshotCloneControllerWatches(mg
 		return err
 	}
 
-	if err := r.addVolumeCloneSourceWatch(datavolumeController); err != nil {
+	if err := r.addVolumeCloneSourceWatch(mgr, datavolumeController); err != nil {
 		return err
 	}
 
@@ -185,11 +185,14 @@ func (r *SnapshotCloneReconciler) syncSnapshotClone(log logr.Logger, req reconci
 	pvcSpec := syncRes.pvcSpec
 	datavolume := syncRes.dvMutated
 
-	pvcPopulated := pvcIsPopulated(pvc, datavolume)
 	staticProvisionPending := checkStaticProvisionPending(pvc, datavolume)
 	_, prePopulated := datavolume.Annotations[cc.AnnPrePopulated]
+	requiresWork, err := r.pvcRequiresWork(pvc, datavolume)
+	if err != nil {
+		return syncRes, err
+	}
 
-	if pvcPopulated || prePopulated || staticProvisionPending {
+	if !requiresWork || prePopulated || staticProvisionPending {
 		return syncRes, nil
 	}
 
@@ -299,6 +302,9 @@ func (r *SnapshotCloneReconciler) detectCloneSize(log logr.Logger, syncState *dv
 		return false, nil
 	}
 
+	if pvcSpec.Resources.Requests == nil {
+		pvcSpec.Resources.Requests = corev1.ResourceList{}
+	}
 	pvcSpec.Resources.Requests[corev1.ResourceStorage] = *snapshot.Status.RestoreSize
 
 	log.V(3).Info("set pvc request size", "size", pvcSpec.Resources.Requests[corev1.ResourceStorage])
