@@ -32,11 +32,10 @@ import (
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
@@ -133,7 +132,7 @@ var _ = Describe("All DataVolume Tests", func() {
 			Expect(pvc.Annotations[AnnCloneRequest]).To(Equal(fmt.Sprintf("%s/%s", snapshot.Namespace, tempPvcName)))
 			By("Mark target PVC bound like it would be in a live cluster, so DV status is updated")
 			pvc.Status.Phase = corev1.ClaimBound
-			err = reconciler.client.Update(context.TODO(), pvc)
+			err = reconciler.client.Status().Update(context.TODO(), pvc)
 			Expect(err).ToNot(HaveOccurred())
 			_, err = reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "test-dv", Namespace: metav1.NamespaceDefault}})
 			Expect(err).ToNot(HaveOccurred())
@@ -342,7 +341,7 @@ var _ = Describe("All DataVolume Tests", func() {
 					}
 					pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 						Kind:       "DataVolume",
-						Controller: pointer.Bool(true),
+						Controller: ptr.To[bool](true),
 						Name:       "test-dv",
 						UID:        dv.UID,
 					})
@@ -353,7 +352,7 @@ var _ = Describe("All DataVolume Tests", func() {
 						},
 						Spec: cdiv1.VolumeCloneSourceSpec{
 							Source: corev1.TypedLocalObjectReference{
-								APIGroup: pointer.String("snapshot.storage.k8s.io"),
+								APIGroup: ptr.To[string]("snapshot.storage.k8s.io"),
 								Kind:     "VolumeSnapshot",
 								Name:     dv.Spec.Source.Snapshot.Name,
 							},
@@ -381,7 +380,7 @@ var _ = Describe("All DataVolume Tests", func() {
 					pvc := CreatePvcInStorageClass("test-dv", metav1.NamespaceDefault, &scName, anno, nil, corev1.ClaimPending)
 					pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 						Kind:       "DataVolume",
-						Controller: pointer.Bool(true),
+						Controller: ptr.To[bool](true),
 						Name:       "test-dv",
 						UID:        dv.UID,
 					})
@@ -423,7 +422,7 @@ var _ = Describe("All DataVolume Tests", func() {
 					}
 					pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 						Kind:       "DataVolume",
-						Controller: pointer.Bool(true),
+						Controller: ptr.To[bool](true),
 						Name:       "test-dv",
 						UID:        dv.UID,
 					})
@@ -434,7 +433,7 @@ var _ = Describe("All DataVolume Tests", func() {
 						},
 						Spec: cdiv1.VolumeCloneSourceSpec{
 							Source: corev1.TypedLocalObjectReference{
-								APIGroup: pointer.String("snapshot.storage.k8s.io"),
+								APIGroup: ptr.To[string]("snapshot.storage.k8s.io"),
 								Kind:     "VolumeSnapshot",
 								Name:     dv.Spec.Source.Snapshot.Name,
 							},
@@ -483,7 +482,7 @@ var _ = Describe("All DataVolume Tests", func() {
 					}
 					pvc.OwnerReferences = append(pvc.OwnerReferences, metav1.OwnerReference{
 						Kind:       "DataVolume",
-						Controller: pointer.Bool(true),
+						Controller: ptr.To[bool](true),
 						Name:       "test-dv",
 						UID:        dv.UID,
 					})
@@ -494,7 +493,7 @@ var _ = Describe("All DataVolume Tests", func() {
 						},
 						Spec: cdiv1.VolumeCloneSourceSpec{
 							Source: corev1.TypedLocalObjectReference{
-								APIGroup: pointer.String("snapshot.storage.k8s.io"),
+								APIGroup: ptr.To[string]("snapshot.storage.k8s.io"),
 								Kind:     "VolumeSnapshot",
 								Name:     dv.Spec.Source.Snapshot.Name,
 							},
@@ -514,36 +513,36 @@ var _ = Describe("All DataVolume Tests", func() {
 	})
 })
 
-func createSnapshotCloneReconcilerWFFCDisabled(objects ...runtime.Object) *SnapshotCloneReconciler {
+func createSnapshotCloneReconcilerWFFCDisabled(objects ...client.Object) *SnapshotCloneReconciler {
 	cdiConfig := MakeEmptyCDIConfigSpec(common.ConfigName)
 	cdiConfig.Status = cdiv1.CDIConfigStatus{
 		ScratchSpaceStorageClass: testStorageClass,
 	}
 	cdiConfig.Spec.FeatureGates = []string{}
 
-	objs := []runtime.Object{}
+	objs := []client.Object{}
 	objs = append(objs, objects...)
 	objs = append(objs, cdiConfig)
 
 	return createSnapshotCloneReconcilerWithoutConfig(objs...)
 }
 
-func createSnapshotCloneReconciler(objects ...runtime.Object) *SnapshotCloneReconciler {
+func createSnapshotCloneReconciler(objects ...client.Object) *SnapshotCloneReconciler {
 	cdiConfig := MakeEmptyCDIConfigSpec(common.ConfigName)
 	cdiConfig.Status = cdiv1.CDIConfigStatus{
 		ScratchSpaceStorageClass: testStorageClass,
 	}
 	cdiConfig.Spec.FeatureGates = []string{featuregates.HonorWaitForFirstConsumer}
 
-	objs := []runtime.Object{}
+	objs := []client.Object{}
 	objs = append(objs, objects...)
 	objs = append(objs, cdiConfig)
 
 	return createSnapshotCloneReconcilerWithoutConfig(objs...)
 }
 
-func createSnapshotCloneReconcilerWithoutConfig(objects ...runtime.Object) *SnapshotCloneReconciler {
-	objs := []runtime.Object{}
+func createSnapshotCloneReconcilerWithoutConfig(objects ...client.Object) *SnapshotCloneReconciler {
+	objs := []client.Object{}
 	objs = append(objs, objects...)
 
 	// Register operator types with the runtime scheme.
@@ -556,7 +555,8 @@ func createSnapshotCloneReconcilerWithoutConfig(objects ...runtime.Object) *Snap
 
 	builder := fake.NewClientBuilder().
 		WithScheme(s).
-		WithRuntimeObjects(objs...)
+		WithObjects(objs...).
+		WithStatusSubresource(objs...)
 
 	for _, ia := range getIndexArgs() {
 		builder = builder.WithIndex(ia.obj, ia.field, ia.extractValue)
@@ -584,7 +584,7 @@ func createSnapshotCloneReconcilerWithoutConfig(objects ...runtime.Object) *Snap
 			shortTokenValidator: &FakeValidator{Match: "foobar"},
 			longTokenValidator:  &FakeValidator{Match: "foobar", Params: map[string]string{"uid": "uid"}},
 			tokenGenerator:      &FakeGenerator{token: "foobar"},
-			cloneSourceAPIGroup: pointer.String("snapshot.storage.k8s.io"),
+			cloneSourceAPIGroup: ptr.To[string]("snapshot.storage.k8s.io"),
 			cloneSourceKind:     "VolumeSnapshot",
 		},
 	}
