@@ -39,7 +39,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
@@ -234,7 +234,7 @@ var _ = Describe("Import populator tests", func() {
 		})
 
 		DescribeTable("Should create PVC Prime with proper import annotations", func(key, value, expectedValue string) {
-			targetPvc := CreatePvcInStorageClass(targetPvcName, metav1.NamespaceDefault, &sc.Name, map[string]string{}, nil, corev1.ClaimBound)
+			targetPvc := CreatePvcInStorageClass(targetPvcName, metav1.NamespaceDefault, &sc.Name, map[string]string{}, nil, corev1.ClaimPending)
 			targetPvc.Spec.DataSourceRef = dataSourceRef
 			targetPvc.Annotations[key] = value
 			volumeImportSource := getVolumeImportSource(true, metav1.NamespaceDefault)
@@ -275,7 +275,8 @@ var _ = Describe("Import populator tests", func() {
 			Entry("Invalid extra annotation is not passed", "invalid", "test", ""),
 			Entry("Priority class is passed", AnnPriorityClassName, "test", "test"),
 			Entry("pod network is passed", AnnPodNetwork, "test", "test"),
-			Entry("side car injection is passed", AnnPodSidecarInjection, AnnPodSidecarInjectionDefault, AnnPodSidecarInjectionDefault),
+			Entry("istio side car injection is passed", AnnPodSidecarInjectionIstio, AnnPodSidecarInjectionIstioDefault, AnnPodSidecarInjectionIstioDefault),
+			Entry("linkerd side car injection is passed", AnnPodSidecarInjectionLinkerd, AnnPodSidecarInjectionLinkerdDefault, AnnPodSidecarInjectionLinkerdDefault),
 			Entry("multus default network is passed", AnnPodMultusDefaultNetwork, "test", "test"),
 			Entry("retain pod annotation is passed", AnnPodRetainAfterCompletion, "true", "true"),
 		)
@@ -394,8 +395,58 @@ var _ = Describe("Import populator tests", func() {
 			Entry("with pod succeded phase", string(corev1.PodSucceeded)),
 		)
 
+		It("should update target pvc with desired labels from succeeded pvc prime", func() {
+			const (
+				testKubevirtIoKey           = "test.kubevirt.io/test"
+				testKubevirtIoValue         = "testvalue"
+				testKubevirtIoKeyExisting   = "test.kubevirt.io/existing"
+				testKubevirtIoValueExisting = "existing"
+				testUndesiredKey            = "undesired.key"
+			)
+
+			// The existing key should not be overwritten
+			targetPvc := CreatePvcInStorageClass(targetPvcName, metav1.NamespaceDefault, &sc.Name, nil,
+				map[string]string{testKubevirtIoKeyExisting: testKubevirtIoValueExisting}, corev1.ClaimPending)
+			targetPvc.Spec.DataSourceRef = dataSourceRef
+			volumeImportSource := getVolumeImportSource(true, metav1.NamespaceDefault)
+			pvcPrime := getPVCPrime(targetPvc, nil)
+			AddAnnotation(pvcPrime, AnnPodPhase, string(corev1.PodSucceeded))
+
+			AddLabel(pvcPrime, testKubevirtIoKey, testKubevirtIoValue)
+			AddLabel(pvcPrime, testKubevirtIoKeyExisting, "somethingelse")
+			AddLabel(pvcPrime, testUndesiredKey, testKubevirtIoValue)
+
+			pv := &corev1.PersistentVolume{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "pv",
+				},
+				Spec: corev1.PersistentVolumeSpec{
+					ClaimRef: &corev1.ObjectReference{
+						Namespace: pvcPrime.Namespace,
+						Name:      pvcPrime.Name,
+					},
+				},
+			}
+			pvcPrime.Spec.VolumeName = pv.Name
+
+			By("Reconcile")
+			reconciler = createImportPopulatorReconciler(targetPvc, pvcPrime, pv, volumeImportSource, sc)
+			result, err := reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: targetPvcName, Namespace: metav1.NamespaceDefault}})
+			Expect(err).To(Not(HaveOccurred()))
+			Expect(result).ToNot(BeNil())
+			Expect(result.RequeueAfter).To(Equal(0 * time.Second))
+
+			updatedPVC := &corev1.PersistentVolumeClaim{}
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: targetPvcName, Namespace: metav1.NamespaceDefault}, updatedPVC)
+			Expect(err).ToNot(HaveOccurred())
+
+			Expect(updatedPVC.Labels).To(HaveKeyWithValue(testKubevirtIoKey, testKubevirtIoValue))
+			Expect(updatedPVC.Labels).To(HaveKeyWithValue(testKubevirtIoKeyExisting, testKubevirtIoValueExisting))
+			Expect(updatedPVC.Labels).ToNot(HaveKey(testUndesiredKey))
+		})
+
 		It("Should set multistage migration annotations on PVC prime", func() {
-			targetPvc := CreatePvcInStorageClass(targetPvcName, metav1.NamespaceDefault, &sc.Name, nil, nil, corev1.ClaimBound)
+			targetPvc := CreatePvcInStorageClass(targetPvcName, metav1.NamespaceDefault, &sc.Name, nil, nil, corev1.ClaimPending)
 			targetPvc.Spec.DataSourceRef = dataSourceRef
 			volumeImportSource := getVolumeImportSource(true, metav1.NamespaceDefault)
 			volumeImportSource.Spec.Source = &cdiv1.ImportSourceType{
@@ -413,7 +464,7 @@ var _ = Describe("Import populator tests", func() {
 					Current:  "current",
 				},
 			}
-			volumeImportSource.Spec.FinalCheckpoint = pointer.Bool(true)
+			volumeImportSource.Spec.FinalCheckpoint = ptr.To[bool](true)
 
 			By("Reconcile")
 			reconciler = createImportPopulatorReconciler(targetPvc, volumeImportSource, sc)
