@@ -195,7 +195,7 @@ var _ = Describe("Datavolume controller reconcile loop", func() {
 		cc.AddAnnotation(pvcPrime, AnnPVCPrimeName, pvcPrime.Name)
 
 		r := createUploadPopulatorReconciler(pvc, pvcPrime)
-		err := r.updatePVCWithPVCPrimeAnnotations(pvc, pvcPrime, r.updateUploadAnnotations)
+		pvc, err := r.updatePVCWithPVCPrimeAnnotations(pvc, pvcPrime, r.updateUploadAnnotations)
 		Expect(err).ToNot(HaveOccurred())
 
 		//should always remove the pvc prime annotation
@@ -330,7 +330,8 @@ var _ = Describe("Datavolume controller reconcile loop", func() {
 		Entry("Invalid extra annotation is not passed", "invalid", "test", ""),
 		Entry("Priority class is passed", cc.AnnPriorityClassName, "test", "test"),
 		Entry("pod network is passed", cc.AnnPodNetwork, "test", "test"),
-		Entry("side car injection is passed", cc.AnnPodSidecarInjection, cc.AnnPodSidecarInjectionDefault, cc.AnnPodSidecarInjectionDefault),
+		Entry("istio side car injection is passed", cc.AnnPodSidecarInjectionIstio, cc.AnnPodSidecarInjectionIstioDefault, cc.AnnPodSidecarInjectionIstioDefault),
+		Entry("linkerd side car injection is passed", cc.AnnPodSidecarInjectionLinkerd, cc.AnnPodSidecarInjectionLinkerdDefault, cc.AnnPodSidecarInjectionLinkerdDefault),
 		Entry("multus default network is passed", cc.AnnPodMultusDefaultNetwork, "test", "test"),
 	)
 })
