@@ -19,7 +19,6 @@ package controller
 import (
 	"context"
 	"strings"
-	"time"
 
 	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
 	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
@@ -180,15 +179,16 @@ var _ = Describe("Upload controller reconcile loop", func() {
 
 	It("Should return nil and remove any service and pod if pvc marked for deletion", func() {
 		testPvc := cc.CreatePvc("testPvc1", "default", map[string]string{cc.AnnUploadRequest: "", cc.AnnPodPhase: string(corev1.PodPending)}, nil)
-		now := metav1.NewTime(time.Now())
-		testPvc.DeletionTimestamp = &now
+		testPvc.Finalizers = append(testPvc.Finalizers, "keepmearound")
 		reconciler := createUploadReconciler(testPvc,
 			createUploadPod(testPvc),
 			createUploadService(testPvc),
 		)
+		err := reconciler.client.Delete(context.TODO(), testPvc)
+		Expect(err).ToNot(HaveOccurred())
 		By("Verifying the pod and service exists")
 		uploadPod := &corev1.Pod{}
-		err := reconciler.client.Get(context.TODO(), types.NamespacedName{Name: createUploadResourceName("testPvc1"), Namespace: "default"}, uploadPod)
+		err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: createUploadResourceName("testPvc1"), Namespace: "default"}, uploadPod)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(uploadPod.Name).To(Equal(createUploadResourceName(testPvc.Name)))
 
@@ -334,7 +334,8 @@ var _ = Describe("reconcilePVC loop", func() {
 			Expect(uploadPod.Name).To(Equal(uploadResourceName))
 			Expect(uploadPod.Labels[common.UploadTargetLabel]).To(Equal(string(testPvc.UID)))
 			Expect(uploadPod.GetAnnotations()[cc.AnnPodNetwork]).To(Equal("net1"))
-			Expect(uploadPod.GetAnnotations()[cc.AnnPodSidecarInjection]).To(Equal(cc.AnnPodSidecarInjectionDefault))
+			Expect(uploadPod.GetAnnotations()[cc.AnnPodSidecarInjectionIstio]).To(Equal(cc.AnnPodSidecarInjectionIstioDefault))
+			Expect(uploadPod.GetAnnotations()[cc.AnnPodSidecarInjectionLinkerd]).To(Equal(cc.AnnPodSidecarInjectionLinkerdDefault))
 
 			uploadService = &corev1.Service{}
 			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: naming.GetServiceNameFromResourceName(uploadResourceName), Namespace: "default"}, uploadService)
@@ -594,7 +595,7 @@ var _ = Describe("updateUploadAnnotations", func() {
 
 		pvcCopy := testPvc.DeepCopy()
 
-		updateUploadAnnotations(testPvc, pvcCopy.Annotations, pod, false)
+		updateUploadAnnotations(pvcCopy.Annotations, pod)
 		Expect(pvcCopy.Annotations[cc.AnnPodRestarts]).To(Equal("1"))
 		Expect(pvcCopy.GetAnnotations()[cc.AnnRunningCondition]).To(Equal("true"))
 		Expect(pvcCopy.GetAnnotations()[cc.AnnRunningConditionMessage]).To(Equal(""))
