@@ -12,6 +12,7 @@ import (
 
 	"github.com/pkg/errors"
 
+	"kubevirt.io/containerized-data-importer/pkg/common"
 	"kubevirt.io/containerized-data-importer/pkg/image"
 )
 
@@ -89,6 +90,11 @@ func (m *MockDataProvider) GetURL() *url.URL {
 	return m.url
 }
 
+// GetTerminationMessage returns data to be serialized and used as the termination message of the importer.
+func (m *MockDataProvider) GetTerminationMessage() *common.TerminationMessage {
+	return nil
+}
+
 // Close closes any readers or other open resources.
 func (m *MockDataProvider) Close() error {
 	return nil
