@@ -38,6 +38,7 @@ import (
 	"github.com/pkg/errors"
 	"k8s.io/klog/v2"
 
+	"kubevirt.io/containerized-data-importer/pkg/common"
 	"kubevirt.io/containerized-data-importer/pkg/util"
 )
 
@@ -193,6 +194,11 @@ func (is *ImageioDataSource) GetURL() *url.URL {
 	return is.url
 }
 
+// GetTerminationMessage returns data to be serialized and used as the termination message of the importer.
+func (is *ImageioDataSource) GetTerminationMessage() *common.TerminationMessage {
+	return nil
+}
+
 // Close all readers.
 func (is *ImageioDataSource) Close() error {
 	var err error
