@@ -7,7 +7,6 @@ import (
 	"encoding/pem"
 	"fmt"
 	"io"
-	"k8s.io/klog/v2"
 	"net/http"
 	"net/http/httptest"
 	"os"
@@ -16,6 +15,8 @@ import (
 	"strings"
 	"time"
 
+	"k8s.io/klog/v2"
+
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	ovirtsdk4 "github.com/ovirt/go-ovirt"
@@ -46,7 +47,7 @@ var _ = Describe("Imageio reader", func() {
 		newOvirtClientFunc = createMockOvirtClient
 		newTerminationChannel = createMockTerminationChannel
 		tempDir = createCert()
-		ts = createTestServer(imageDir)
+		ts = createTestServer(imageDir, nil)
 		disk.SetTotalSize(1024)
 		disk.SetId(diskID)
 		it.SetPhase(ovirtsdk4.IMAGETRANSFERPHASE_TRANSFERRING)
@@ -91,7 +92,7 @@ var _ = Describe("Imageio data source", func() {
 		newOvirtClientFunc = createMockOvirtClient
 		newTerminationChannel = createMockTerminationChannel
 		tempDir = createCert()
-		ts = createTestServer(imageDir)
+		ts = createTestServer(imageDir, nil)
 		disk.SetTotalSize(1024)
 		disk.SetId(diskID)
 		it.SetPhase(ovirtsdk4.IMAGETRANSFERPHASE_TRANSFERRING)
@@ -244,7 +245,7 @@ var _ = Describe("Imageio cancel", func() {
 		newOvirtClientFunc = createMockOvirtClient
 		newTerminationChannel = createMockTerminationChannel
 		tempDir = createCert()
-		ts = createTestServer(imageDir)
+		ts = createTestServer(imageDir, nil)
 		disk.SetTotalSize(1024)
 		disk.SetId(diskID)
 		it.SetPhase(ovirtsdk4.IMAGETRANSFERPHASE_TRANSFERRING)
@@ -384,7 +385,7 @@ var _ = Describe("imageio snapshots", func() {
 		newOvirtClientFunc = createMockOvirtClient
 		newTerminationChannel = createMockTerminationChannel
 		tempDir = createCert()
-		ts = createTestServer(imageDir)
+		ts = createTestServer(imageDir, nil)
 		disk.SetTotalSize(diskSize)
 		disk.SetId(diskID)
 		it.SetPhase(ovirtsdk4.IMAGETRANSFERPHASE_TRANSFERRING)
