@@ -24,11 +24,9 @@ import (
 	"reflect"
 	"strconv"
 	"strings"
-	"time"
-
-	"kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/callbacks"
 
 	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/api"
+	"kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/callbacks"
 	sdkr "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/reconciler"
 
 	. "github.com/onsi/ginkgo/v2"
@@ -50,6 +48,7 @@ import (
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
+	"sigs.k8s.io/controller-runtime/pkg/cache"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	fakeClient "sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
@@ -57,6 +56,8 @@ import (
 
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
+	"kubevirt.io/containerized-data-importer/pkg/monitoring/rules"
+	"kubevirt.io/containerized-data-importer/pkg/monitoring/rules/alerts"
 	clusterResources "kubevirt.io/containerized-data-importer/pkg/operator/resources/cluster"
 	namespaceResources "kubevirt.io/containerized-data-importer/pkg/operator/resources/namespaced"
 	utils "kubevirt.io/containerized-data-importer/pkg/operator/resources/utils"
@@ -285,7 +286,7 @@ var _ = Describe("Controller", func() {
 				doReconcile(args)
 				Expect(setDeploymentsReady(args)).To(BeTrue())
 
-				runbookURLTemplate := getRunbookURLTemplate()
+				runbookURLTemplate := alerts.GetRunbookURLTemplate()
 
 				rule := &promv1.PrometheusRule{
 					ObjectMeta: metav1.ObjectMeta{
@@ -313,7 +314,7 @@ var _ = Describe("Controller", func() {
 					},
 				}
 
-				Expect(rule.Spec.Groups[0].Rules).To(ContainElement(cdiDownAlert))
+				Expect(rule.Spec.Groups[1].Rules).To(ContainElement(cdiDownAlert))
 				Expect(rule.Labels[common.AppKubernetesPartOfLabel]).To(Equal("testing"))
 				validateEvents(args.reconciler, createReadyEventValidationMap())
 			})
@@ -512,6 +513,21 @@ var _ = Describe("Controller", func() {
 				Expect(cm.Labels[common.AppKubernetesPartOfLabel]).To(Equal("newtesting"))
 			})
 
+			It("should get rid of deprecated insecure registries config map", func() {
+				args := createArgs()
+				doReconcile(args)
+				Expect(setDeploymentsReady(args)).To(BeTrue())
+
+				cm := &corev1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Namespace: cdiNamespace, Name: "cdi-insecure-registries"}}
+				err := args.client.Create(context.TODO(), cm)
+				Expect(err).ToNot(HaveOccurred())
+
+				doReconcile(args)
+
+				_, err = getObject(args.client, cm)
+				Expect(errors.IsNotFound(err)).To(BeTrue())
+			})
+
 			It("should set config authority", func() {
 				args := createArgs()
 				doReconcile(args)
@@ -691,8 +707,7 @@ var _ = Describe("Controller", func() {
 				err := args.client.Create(context.TODO(), pod)
 				Expect(err).ToNot(HaveOccurred())
 
-				args.cdi.DeletionTimestamp = &metav1.Time{Time: time.Now()}
-				err = args.client.Update(context.TODO(), args.cdi)
+				err = args.client.Delete(context.TODO(), args.cdi)
 				Expect(err).ToNot(HaveOccurred())
 
 				doReconcileExpectDelete(args)
@@ -852,10 +867,10 @@ var _ = Describe("Controller", func() {
 
 					//Modify CRD to be of previousVersion
 					Expect(crSetVersion(args.reconciler.reconciler, args.cdi, prevVersion)).To(Succeed())
-					//marc CDI CR for deltetion
-					args.cdi.SetDeletionTimestamp(&metav1.Time{Time: time.Now()})
+					//mark CDI CR for deletion
 					args.cdi.Finalizers = append(args.cdi.Finalizers, "keepmearound")
 					Expect(args.client.Update(context.TODO(), args.cdi)).To(Succeed())
+					Expect(args.client.Delete(context.TODO(), args.cdi)).To(Succeed())
 
 					doReconcile(args)
 
@@ -885,9 +900,8 @@ var _ = Describe("Controller", func() {
 					//begin upgrade
 					doReconcile(args)
 
-					//mark CDI CR for deltetion
-					args.cdi.SetDeletionTimestamp(&metav1.Time{Time: time.Now()})
-					Expect(args.client.Update(context.TODO(), args.cdi)).To(Succeed())
+					//mark CDI CR for deletion
+					Expect(args.client.Delete(context.TODO(), args.cdi)).To(Succeed())
 
 					doReconcileExpectDelete(args)
 
@@ -1724,6 +1738,11 @@ func createReconciler(client client.Client) *ReconcileCDI {
 		Namespace:              namespace,
 	}
 
+	err := rules.SetupRules(namespace)
+	if err != nil {
+		panic(err)
+	}
+
 	recorder := record.NewFakeRecorder(250)
 	r := &ReconcileCDI{
 		client:         client,
@@ -1736,7 +1755,10 @@ func createReconciler(client client.Client) *ReconcileCDI {
 		certManager:    newFakeCertManager(client, namespace),
 	}
 	callbackDispatcher := callbacks.NewCallbackDispatcher(log, client, client, scheme.Scheme, namespace)
-	r.reconciler = sdkr.NewReconciler(r, log, client, callbackDispatcher, scheme.Scheme, createVersionLabel, updateVersionLabel, LastAppliedConfigAnnotation, certPollInterval, finalizerName, false, recorder).
+	getCache := func() cache.Cache {
+		return nil
+	}
+	r.reconciler = sdkr.NewReconciler(r, log, client, callbackDispatcher, scheme.Scheme, getCache, createVersionLabel, updateVersionLabel, LastAppliedConfigAnnotation, certPollInterval, finalizerName, false, recorder).
 		WithWatching(true)
 
 	r.registerHooks()
