@@ -30,12 +30,13 @@ import (
 
 	cdicorev1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	cdiuploadv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/upload/v1beta1"
+	"kubevirt.io/containerized-data-importer/pkg/common"
 	"kubevirt.io/containerized-data-importer/pkg/operator/resources/utils"
 )
 
 const (
-	apiServerResourceName = "cdi-apiserver"
-	apiServerServiceName  = "cdi-api"
+	// APIServerServiceName is the API server service name
+	APIServerServiceName = "cdi-api"
 )
 
 func createStaticAPIServerResources(args *FactoryArgs) []client.Object {
@@ -92,8 +93,39 @@ func getAPIServerClusterPolicyRules() []rbacv1.PolicyRule {
 			},
 			Verbs: []string{
 				"get",
+				"list",
+				"watch",
 			},
 		},
+
+		{
+			APIGroups: []string{
+				"",
+			},
+			Resources: []string{
+				"persistentvolumes",
+			},
+			Verbs: []string{
+				"get",
+				"list",
+				"watch",
+			},
+		},
+
+		{
+			APIGroups: []string{
+				"storage.k8s.io",
+			},
+			Resources: []string{
+				"storageclasses",
+			},
+			Verbs: []string{
+				"get",
+				"list",
+				"watch",
+			},
+		},
+
 		{
 			APIGroups: []string{
 				"",
@@ -114,6 +146,8 @@ func getAPIServerClusterPolicyRules() []rbacv1.PolicyRule {
 			},
 			Verbs: []string{
 				"get",
+				"list",
+				"watch",
 			},
 		},
 		{
@@ -139,6 +173,35 @@ func getAPIServerClusterPolicyRules() []rbacv1.PolicyRule {
 				"get",
 			},
 		},
+
+		{
+			APIGroups: []string{
+				"cdi.kubevirt.io",
+			},
+			Resources: []string{
+				"volumeclonesources",
+			},
+			Verbs: []string{
+				"get",
+				"list",
+				"watch",
+			},
+		},
+
+		{
+			APIGroups: []string{
+				"cdi.kubevirt.io",
+			},
+			Resources: []string{
+				"storageprofiles",
+			},
+			Verbs: []string{
+				"get",
+				"list",
+				"watch",
+			},
+		},
+
 		{
 			APIGroups: []string{
 				"cdi.kubevirt.io",
@@ -148,6 +211,21 @@ func getAPIServerClusterPolicyRules() []rbacv1.PolicyRule {
 			},
 			Verbs: []string{
 				"get",
+				"list",
+				"watch",
+			},
+		},
+		{
+			APIGroups: []string{
+				"cdi.kubevirt.io",
+			},
+			Resources: []string{
+				"cdiconfigs",
+			},
+			Verbs: []string{
+				"get",
+				"list",
+				"watch",
 			},
 		},
 		{
@@ -173,13 +251,13 @@ func createAPIService(version, namespace string, c client.Client, l logr.Logger)
 		ObjectMeta: metav1.ObjectMeta{
 			Name: fmt.Sprintf("%s.%s", version, cdiuploadv1.SchemeGroupVersion.Group),
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Spec: apiregistrationv1.APIServiceSpec{
 			Service: &apiregistrationv1.ServiceReference{
 				Namespace: namespace,
-				Name:      apiServerServiceName,
+				Name:      APIServerServiceName,
 			},
 			Group:                cdiuploadv1.SchemeGroupVersion.Group,
 			Version:              version,
@@ -192,7 +270,7 @@ func createAPIService(version, namespace string, c client.Client, l logr.Logger)
 		return apiService
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		apiService.Spec.CABundle = bundle
 	}
@@ -216,7 +294,7 @@ func createDataImportCronValidatingWebhook(namespace string, c client.Client, l
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "cdi-api-dataimportcron-validate",
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Webhooks: []admissionregistrationv1.ValidatingWebhook{
@@ -237,7 +315,7 @@ func createDataImportCronValidatingWebhook(namespace string, c client.Client, l
 				ClientConfig: admissionregistrationv1.WebhookClientConfig{
 					Service: &admissionregistrationv1.ServiceReference{
 						Namespace: namespace,
-						Name:      apiServerServiceName,
+						Name:      APIServerServiceName,
 						Path:      &path,
 						Port:      &defaultServicePort,
 					},
@@ -259,7 +337,7 @@ func createDataImportCronValidatingWebhook(namespace string, c client.Client, l
 		return whc
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		whc.Webhooks[0].ClientConfig.CABundle = bundle
 	}
@@ -283,7 +361,7 @@ func createPopulatorsValidatingWebhook(namespace string, c client.Client, l logr
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "cdi-api-populator-validate",
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Webhooks: []admissionregistrationv1.ValidatingWebhook{
@@ -304,7 +382,7 @@ func createPopulatorsValidatingWebhook(namespace string, c client.Client, l logr
 				ClientConfig: admissionregistrationv1.WebhookClientConfig{
 					Service: &admissionregistrationv1.ServiceReference{
 						Namespace: namespace,
-						Name:      apiServerServiceName,
+						Name:      APIServerServiceName,
 						Path:      &path,
 						Port:      &defaultServicePort,
 					},
@@ -326,7 +404,7 @@ func createPopulatorsValidatingWebhook(namespace string, c client.Client, l logr
 		return whc
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		whc.Webhooks[0].ClientConfig.CABundle = bundle
 	}
@@ -350,7 +428,7 @@ func createDataVolumeValidatingWebhook(namespace string, c client.Client, l logr
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "cdi-api-datavolume-validate",
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Webhooks: []admissionregistrationv1.ValidatingWebhook{
@@ -371,7 +449,7 @@ func createDataVolumeValidatingWebhook(namespace string, c client.Client, l logr
 				ClientConfig: admissionregistrationv1.WebhookClientConfig{
 					Service: &admissionregistrationv1.ServiceReference{
 						Namespace: namespace,
-						Name:      apiServerServiceName,
+						Name:      APIServerServiceName,
 						Path:      &path,
 						Port:      &defaultServicePort,
 					},
@@ -393,7 +471,7 @@ func createDataVolumeValidatingWebhook(namespace string, c client.Client, l logr
 		return whc
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		whc.Webhooks[0].ClientConfig.CABundle = bundle
 	}
@@ -407,7 +485,7 @@ func createCDIValidatingWebhook(namespace string, c client.Client, l logr.Logger
 	defaultServicePort := int32(443)
 	allScopes := admissionregistrationv1.AllScopes
 	exactPolicy := admissionregistrationv1.Exact
-	failurePolicy := admissionregistrationv1.Fail
+	failurePolicy := admissionregistrationv1.Ignore
 	defaultTimeoutSeconds := int32(30)
 	whc := &admissionregistrationv1.ValidatingWebhookConfiguration{
 		TypeMeta: metav1.TypeMeta{
@@ -417,7 +495,7 @@ func createCDIValidatingWebhook(namespace string, c client.Client, l logr.Logger
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "cdi-api-validate",
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Webhooks: []admissionregistrationv1.ValidatingWebhook{
@@ -437,7 +515,7 @@ func createCDIValidatingWebhook(namespace string, c client.Client, l logr.Logger
 				ClientConfig: admissionregistrationv1.WebhookClientConfig{
 					Service: &admissionregistrationv1.ServiceReference{
 						Namespace: namespace,
-						Name:      apiServerServiceName,
+						Name:      APIServerServiceName,
 						Path:      &path,
 						Port:      &defaultServicePort,
 					},
@@ -459,7 +537,7 @@ func createCDIValidatingWebhook(namespace string, c client.Client, l logr.Logger
 		return whc
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		for i := range whc.Webhooks {
 			whc.Webhooks[i].ClientConfig.CABundle = bundle
@@ -486,7 +564,7 @@ func createObjectTransferValidatingWebhook(namespace string, c client.Client, l
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "objecttransfer-api-validate",
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Webhooks: []admissionregistrationv1.ValidatingWebhook{
@@ -509,7 +587,7 @@ func createObjectTransferValidatingWebhook(namespace string, c client.Client, l
 				ClientConfig: admissionregistrationv1.WebhookClientConfig{
 					Service: &admissionregistrationv1.ServiceReference{
 						Namespace: namespace,
-						Name:      apiServerServiceName,
+						Name:      APIServerServiceName,
 						Path:      &path,
 						Port:      &defaultServicePort,
 					},
@@ -531,7 +609,7 @@ func createObjectTransferValidatingWebhook(namespace string, c client.Client, l
 		return whc
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		for i := range whc.Webhooks {
 			whc.Webhooks[i].ClientConfig.CABundle = bundle
@@ -559,7 +637,7 @@ func createDataVolumeMutatingWebhook(namespace string, c client.Client, l logr.L
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "cdi-api-datavolume-mutate",
 			Labels: map[string]string{
-				utils.CDILabel: apiServerServiceName,
+				utils.CDILabel: APIServerServiceName,
 			},
 		},
 		Webhooks: []admissionregistrationv1.MutatingWebhook{
@@ -580,7 +658,7 @@ func createDataVolumeMutatingWebhook(namespace string, c client.Client, l logr.L
 				ClientConfig: admissionregistrationv1.WebhookClientConfig{
 					Service: &admissionregistrationv1.ServiceReference{
 						Namespace: namespace,
-						Name:      apiServerServiceName,
+						Name:      APIServerServiceName,
 						Path:      &path,
 						Port:      &defaultServicePort,
 					},
@@ -603,7 +681,7 @@ func createDataVolumeMutatingWebhook(namespace string, c client.Client, l logr.L
 		return whc
 	}
 
-	bundle := getAPIServerCABundle(namespace, c, l)
+	bundle := GetAPIServerCABundle(namespace, c, l)
 	if bundle != nil {
 		whc.Webhooks[0].ClientConfig.CABundle = bundle
 	}
@@ -611,7 +689,8 @@ func createDataVolumeMutatingWebhook(namespace string, c client.Client, l logr.L
 	return whc
 }
 
-func getAPIServerCABundle(namespace string, c client.Client, l logr.Logger) []byte {
+// GetAPIServerCABundle returns the API server CA bundle
+func GetAPIServerCABundle(namespace string, c client.Client, l logr.Logger) []byte {
 	cm := &corev1.ConfigMap{}
 	key := client.ObjectKey{Namespace: namespace, Name: "cdi-apiserver-signer-bundle"}
 	if err := c.Get(context.TODO(), key, cm); err != nil {
@@ -626,9 +705,9 @@ func getAPIServerCABundle(namespace string, c client.Client, l logr.Logger) []by
 }
 
 func createAPIServerClusterRoleBinding(namespace string) *rbacv1.ClusterRoleBinding {
-	return utils.ResourceBuilder.CreateClusterRoleBinding(apiServerResourceName, apiServerResourceName, apiServerResourceName, namespace)
+	return utils.ResourceBuilder.CreateClusterRoleBinding(common.CDIApiServerResourceName, common.CDIApiServerResourceName, common.CDIApiServerResourceName, namespace)
 }
 
 func createAPIServerClusterRole() *rbacv1.ClusterRole {
-	return utils.ResourceBuilder.CreateClusterRole(apiServerResourceName, getAPIServerClusterPolicyRules())
+	return utils.ResourceBuilder.CreateClusterRole(common.CDIApiServerResourceName, getAPIServerClusterPolicyRules())
 }
