@@ -35,7 +35,7 @@ const (
 	errorDigest           = "sha256:12345678900987654321"
 )
 
-var _ = Describe("DataImportCron", func() {
+var _ = Describe("DataImportCron", Serial, func() {
 	var (
 		f                   = framework.NewFramework("dataimportcron-func-test")
 		log                 = logf.Log.WithName("dataimportcron_test")
@@ -134,7 +134,16 @@ var _ = Describe("DataImportCron", func() {
 				},
 			}
 			snapshot = utils.WaitSnapshotReady(f.CrClient, snapshot)
-			deleted, err := utils.WaitPVCDeleted(f.K8sClient, name, ns, 2*time.Minute)
+			deleted, err := utils.WaitPVCDeleted(f.K8sClient, name, ns, 30*time.Second)
+			if err != nil {
+				// work around https://github.com/kubernetes-csi/external-snapshotter/issues/957
+				// it does converge after the resync period of snapshot controller (15mins)
+				cc.AddAnnotation(snapshot, "workaround", "triggersync")
+				err = f.CrClient.Update(context.TODO(), snapshot)
+				Expect(err).ToNot(HaveOccurred())
+				// try again
+				deleted, err = utils.WaitPVCDeleted(f.K8sClient, name, ns, 30*time.Second)
+			}
 			Expect(err).ToNot(HaveOccurred())
 			Expect(deleted).To(BeTrue())
 			// check pvc is not recreated
@@ -503,9 +512,11 @@ var _ = Describe("DataImportCron", func() {
 			_, err = f.K8sClient.CoreV1().PersistentVolumeClaims(pvc.Namespace).Update(context.TODO(), pvc, metav1.UpdateOptions{})
 			Expect(err).ToNot(HaveOccurred())
 
-			pvcList, err := f.K8sClient.CoreV1().PersistentVolumeClaims(ns).List(context.TODO(), metav1.ListOptions{})
-			Expect(err).ToNot(HaveOccurred())
-			Expect(pvcList.Items).To(HaveLen(garbageSources + 1))
+			Eventually(func() []corev1.PersistentVolumeClaim {
+				pvcList, err := f.K8sClient.CoreV1().PersistentVolumeClaims(ns).List(context.TODO(), metav1.ListOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				return pvcList.Items
+			}, dataImportCronTimeout, pollingInterval).Should(HaveLen(garbageSources + 1))
 		case cdiv1.DataImportCronSourceFormatSnapshot:
 			snapshots := &snapshotv1.VolumeSnapshotList{}
 			err := f.CrClient.List(context.TODO(), snapshots, &client.ListOptions{Namespace: ns})
