@@ -3,13 +3,10 @@ package framework
 import (
 	"bytes"
 	"context"
-	"encoding/json"
 	"fmt"
 	"os"
 	"os/exec"
-	"path/filepath"
 	"reflect"
-	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -35,6 +32,7 @@ import (
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/clientcmd"
+	storagehelpers "k8s.io/component-helpers/storage/volume"
 	"k8s.io/klog/v2"
 	crclient "sigs.k8s.io/controller-runtime/pkg/client"
 
@@ -52,13 +50,13 @@ const (
 	nsDeleteTime = 5 * time.Minute
 	//NsPrefixLabel provides a cdi prefix label to identify the test namespace
 	NsPrefixLabel = "cdi-e2e"
-	cdiPodPrefix  = "cdi-deployment"
+	//CdiPodPrefix provides the cdi-deployment pod prefix
+	CdiPodPrefix = "cdi-deployment"
 )
 
 // run-time flags
 var (
 	ClientsInstance = &Clients{}
-	reporter        = NewKubernetesReporter()
 )
 
 // Config provides some basic test config options
@@ -126,7 +124,6 @@ type Framework struct {
 	ControllerPod *v1.Pod
 
 	*Clients
-	reporter *KubernetesReporter
 }
 
 // NewFramework calls NewFramework and handles errors by calling Fail. Config is optional, but
@@ -145,7 +142,6 @@ func NewFramework(prefix string, config ...Config) *Framework {
 		Config:   cfg,
 		NsPrefix: prefix,
 		Clients:  ClientsInstance,
-		reporter: reporter,
 	}
 
 	ginkgo.BeforeEach(f.BeforeEach)
@@ -167,7 +163,7 @@ func (f *Framework) BeforeEach() {
 	}
 
 	if f.ControllerPod == nil {
-		pod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, cdiPodPrefix, common.CDILabelSelector)
+		pod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, CdiPodPrefix, common.CDILabelSelector)
 		gomega.Expect(err).NotTo(gomega.HaveOccurred())
 		fmt.Fprintf(ginkgo.GinkgoWriter, "INFO: Located cdi-controller-pod: %q\n", pod.Name)
 		f.ControllerPod = pod
@@ -210,12 +206,6 @@ func (f *Framework) AfterEach() {
 			}
 		}
 	}()
-
-	if ginkgo.CurrentSpecReport().Failed() {
-		f.reporter.FailureCount++
-		fmt.Fprintf(ginkgo.GinkgoWriter, "On failure, artifacts will be collected in %s/%d_*\n", f.reporter.artifactsDir, f.reporter.FailureCount)
-		f.reporter.Dump(f.K8sClient, f.CdiClient, ginkgo.CurrentSpecReport().RunTime)
-	}
 }
 
 // CreateNamespace instantiates a new namespace object with a unique name and the passed-in label(s).
@@ -240,9 +230,9 @@ func (f *Framework) CreateNamespace(prefix string, labels map[string]string) (*v
 
 	var nsObj *v1.Namespace
 	c := f.K8sClient
-	err := wait.PollImmediate(2*time.Second, nsCreateTime, func() (bool, error) {
+	err := wait.PollUntilContextTimeout(context.TODO(), 2*time.Second, nsCreateTime, true, func(ctx context.Context) (bool, error) {
 		var err error
-		nsObj, err = c.CoreV1().Namespaces().Create(context.TODO(), ns, metav1.CreateOptions{})
+		nsObj, err = c.CoreV1().Namespaces().Create(ctx, ns, metav1.CreateOptions{})
 		if err == nil || apierrs.IsAlreadyExists(err) {
 			return true, nil // done
 		}
@@ -264,7 +254,6 @@ func (f *Framework) AddNamespaceToDelete(ns *v1.Namespace) {
 
 // DeleteNS provides a function to delete the specified namespace from the test cluster
 func DeleteNS(c *kubernetes.Clientset, ns string) error {
-	// return wait.PollImmediate(2*time.Second, nsDeleteTime, func() (bool, error) {
 	err := c.CoreV1().Namespaces().Delete(context.TODO(), ns, metav1.DeleteOptions{})
 	if err != nil && !apierrs.IsNotFound(err) {
 		return err
@@ -364,7 +353,6 @@ func (f *Framework) GetTokenForServiceAccount(namespace, name string) (string, e
 	if err != nil {
 		return "", err
 	}
-	fmt.Fprintf(ginkgo.GinkgoWriter, "INFO: Token created for SA: %+v\n", token.Status)
 
 	return token.Status.Token, nil
 }
@@ -457,8 +445,8 @@ func (f *Framework) CreateQuotaInSpecifiedNs(ns string, requestCPU, requestMemor
 	if err != nil {
 		ginkgo.Fail("Unable to set resource quota " + err.Error())
 	}
-	return wait.PollImmediate(2*time.Second, nsDeleteTime, func() (bool, error) {
-		quota, err := f.K8sClient.CoreV1().ResourceQuotas(ns).Get(context.TODO(), "test-quota", metav1.GetOptions{})
+	return wait.PollUntilContextTimeout(context.TODO(), 2*time.Second, nsDeleteTime, true, func(ctx context.Context) (bool, error) {
+		quota, err := f.K8sClient.CoreV1().ResourceQuotas(ns).Get(ctx, "test-quota", metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
@@ -486,8 +474,8 @@ func (f *Framework) UpdateQuotaInNs(requestCPU, requestMemory, limitsCPU, limits
 	if err != nil {
 		ginkgo.Fail("Unable to set resource quota " + err.Error())
 	}
-	return wait.PollImmediate(5*time.Second, nsDeleteTime, func() (bool, error) {
-		quota, err := f.K8sClient.CoreV1().ResourceQuotas(f.Namespace.GetName()).Get(context.TODO(), "test-quota", metav1.GetOptions{})
+	return wait.PollUntilContextTimeout(context.TODO(), 5*time.Second, nsDeleteTime, true, func(ctx context.Context) (bool, error) {
+		quota, err := f.K8sClient.CoreV1().ResourceQuotas(f.Namespace.GetName()).Get(ctx, "test-quota", metav1.GetOptions{})
 		if err != nil {
 			fmt.Fprintf(ginkgo.GinkgoWriter, "ERROR: GET ResourceQuota failed once, retrying: %v\n", err.Error())
 			return false, nil
@@ -515,8 +503,8 @@ func (f *Framework) CreateStorageQuota(numPVCs, requestStorage int64) error {
 	if err != nil {
 		ginkgo.Fail("Unable to set resource quota " + err.Error())
 	}
-	return wait.PollImmediate(2*time.Second, nsDeleteTime, func() (bool, error) {
-		quota, err := f.K8sClient.CoreV1().ResourceQuotas(ns).Get(context.TODO(), "test-storage-quota", metav1.GetOptions{})
+	return wait.PollUntilContextTimeout(context.TODO(), 2*time.Second, nsDeleteTime, true, func(ctx context.Context) (bool, error) {
+		quota, err := f.K8sClient.CoreV1().ResourceQuotas(ns).Get(ctx, "test-storage-quota", metav1.GetOptions{})
 		if err != nil {
 			fmt.Fprintf(ginkgo.GinkgoWriter, "ERROR: GET ResourceQuota failed once, retrying: %v\n", err.Error())
 			return false, nil
@@ -543,8 +531,8 @@ func (f *Framework) UpdateStorageQuota(numPVCs, requestStorage int64) error {
 	if err != nil {
 		ginkgo.Fail("Unable to set resource quota " + err.Error())
 	}
-	return wait.PollImmediate(5*time.Second, nsDeleteTime, func() (bool, error) {
-		quota, err := f.K8sClient.CoreV1().ResourceQuotas(f.Namespace.GetName()).Get(context.TODO(), "test-storage-quota", metav1.GetOptions{})
+	return wait.PollUntilContextTimeout(context.TODO(), 5*time.Second, nsDeleteTime, true, func(ctx context.Context) (bool, error) {
+		quota, err := f.K8sClient.CoreV1().ResourceQuotas(f.Namespace.GetName()).Get(ctx, "test-storage-quota", metav1.GetOptions{})
 		if err != nil {
 			fmt.Fprintf(ginkgo.GinkgoWriter, "ERROR: GET ResourceQuota failed once, retrying: %v\n", err.Error())
 			return false, nil
@@ -557,8 +545,8 @@ func (f *Framework) UpdateStorageQuota(numPVCs, requestStorage int64) error {
 
 // DeleteStorageQuota an existing storage quota in the current test namespace.
 func (f *Framework) DeleteStorageQuota() error {
-	return wait.PollImmediate(3*time.Second, time.Minute, func() (bool, error) {
-		err := f.K8sClient.CoreV1().ResourceQuotas(f.Namespace.GetName()).Delete(context.TODO(), "test-storage-quota", metav1.DeleteOptions{})
+	return wait.PollUntilContextTimeout(context.TODO(), 3*time.Second, time.Minute, true, func(ctx context.Context) (bool, error) {
+		err := f.K8sClient.CoreV1().ResourceQuotas(f.Namespace.GetName()).Delete(ctx, "test-storage-quota", metav1.DeleteOptions{})
 		if err != nil {
 			if apierrs.IsNotFound(err) {
 				return true, nil
@@ -618,7 +606,7 @@ func (f *Framework) UpdateCdiConfigResourceLimits(resourceCPU, resourceMemory, l
 	}
 
 	// see if config got updated
-	return wait.PollImmediate(2*time.Second, nsDeleteTime, func() (bool, error) {
+	return wait.PollUntilContextTimeout(context.TODO(), 2*time.Second, nsDeleteTime, true, func(_ context.Context) (bool, error) {
 		res, err := f.runKubectlCommand("get", "CDIConfig", "config", "-o=jsonpath={.status.defaultPodResourceRequirements..['cpu', 'memory']}")
 		if err != nil {
 			return false, err
@@ -741,7 +729,7 @@ func (f *Framework) GetNoSnapshotStorageClass() *string {
 		return nil
 	}
 	for _, sc := range scs.Items {
-		if sc.Name == "local" || strings.Contains(sc.Provisioner, "no-provisioner") {
+		if sc.Name == "local" || sc.Provisioner == storagehelpers.NotSupportedProvisioner {
 			continue
 		}
 		for _, vsc := range vscs.Items {
@@ -808,331 +796,3 @@ func (f *Framework) updateCDIConfig() {
 		})
 	}, timeout, pollingInterval).Should(gomega.BeNil())
 }
-
-func getMaxFailsFromEnv() int {
-	maxFailsEnv := os.Getenv("REPORTER_MAX_FAILS")
-	if maxFailsEnv == "" {
-		fmt.Fprintf(os.Stderr, "defaulting to 10 reported failures\n")
-		return 10
-	}
-
-	maxFails, err := strconv.Atoi(maxFailsEnv)
-	if err != nil { // if the variable is set with a non int value
-		fmt.Println("Invalid REPORTER_MAX_FAILS variable, defaulting to 10")
-		return 10
-	}
-
-	fmt.Fprintf(os.Stderr, "Number of reported failures[%d]\n", maxFails)
-	return maxFails
-}
-
-// KubernetesReporter is the struct that holds the report info.
-type KubernetesReporter struct {
-	FailureCount int
-	artifactsDir string
-	maxFails     int
-}
-
-// NewKubernetesReporter creates a new instance of the reporter.
-func NewKubernetesReporter() *KubernetesReporter {
-	return &KubernetesReporter{
-		FailureCount: 0,
-		artifactsDir: os.Getenv("ARTIFACTS"),
-		maxFails:     getMaxFailsFromEnv(),
-	}
-}
-
-// Dump dumps the current state of the cluster. The relevant logs are collected starting
-// from the since parameter.
-func (r *KubernetesReporter) Dump(kubeCli *kubernetes.Clientset, cdiClient *cdiClientset.Clientset, since time.Duration) {
-	// If we got not directory, print to stderr
-	if r.artifactsDir == "" {
-		return
-	}
-	fmt.Fprintf(os.Stderr, "Current failure count[%d]\n", r.FailureCount)
-	if r.FailureCount > r.maxFails {
-		return
-	}
-
-	// Can call this as many times as needed, if the directory exists, nothing happens.
-	if err := os.MkdirAll(r.artifactsDir, 0777); err != nil {
-		fmt.Fprintf(os.Stderr, "failed to create directory: %v\n", err)
-		return
-	}
-
-	r.logCSIDrivers(kubeCli)
-	r.logDVs(cdiClient)
-	r.logEvents(kubeCli, since)
-	r.logNodes(kubeCli)
-	r.logPVCs(kubeCli)
-	r.logPVs(kubeCli)
-	r.logPods(kubeCli)
-	r.logServices(kubeCli)
-	r.logEndpoints(kubeCli)
-	r.logLogs(kubeCli, since)
-}
-
-// Cleanup cleans up the current content of the artifactsDir
-func (r *KubernetesReporter) Cleanup() {
-	// clean up artifacts from previous run
-	if r.artifactsDir != "" {
-		os.RemoveAll(r.artifactsDir)
-	}
-}
-
-func (r *KubernetesReporter) logPods(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_pods.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v", err)
-		return
-	}
-	defer f.Close()
-
-	pods, err := kubeCli.CoreV1().Pods(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch pods: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(pods, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logServices(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_services.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v", err)
-		return
-	}
-	defer f.Close()
-
-	services, err := kubeCli.CoreV1().Services(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch services: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(services, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logEndpoints(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_endpoints.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v", err)
-		return
-	}
-	defer f.Close()
-
-	endpoints, err := kubeCli.CoreV1().Endpoints(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch endpointss: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(endpoints, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logNodes(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_nodes.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-		return
-	}
-	defer f.Close()
-
-	nodes, err := kubeCli.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch nodes: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(nodes, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logPVs(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_pvs.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-		return
-	}
-	defer f.Close()
-
-	pvs, err := kubeCli.CoreV1().PersistentVolumes().List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch pvs: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(pvs, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logPVCs(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_pvcs.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-		return
-	}
-	defer f.Close()
-
-	pvcs, err := kubeCli.CoreV1().PersistentVolumeClaims(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch pvcs: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(pvcs, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logDVs(cdiClientset *cdiClientset.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_dvs.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-		return
-	}
-	defer f.Close()
-
-	dvs, err := cdiClientset.CdiV1beta1().DataVolumes(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch datavolumes: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(dvs, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logCSIDrivers(kubeCli *kubernetes.Clientset) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_csidrivers.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-		return
-	}
-	defer f.Close()
-
-	csiDrivers, err := kubeCli.StorageV1().CSIDrivers().List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch csidrivers: %v\n", err)
-		return
-	}
-
-	j, err := json.MarshalIndent(csiDrivers, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
-
-func (r *KubernetesReporter) logLogs(kubeCli *kubernetes.Clientset, since time.Duration) {
-	logsdir := filepath.Join(r.artifactsDir, "pods")
-
-	if err := os.MkdirAll(logsdir, 0777); err != nil {
-		fmt.Fprintf(os.Stderr, "failed to create directory: %v\n", err)
-		return
-	}
-
-	startTime := time.Now().Add(-since).Add(-5 * time.Second)
-
-	pods, err := kubeCli.CoreV1().Pods(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to fetch pods: %v\n", err)
-		return
-	}
-
-	for _, pod := range pods.Items {
-		for _, container := range pod.Spec.Containers {
-			current, err := os.OpenFile(filepath.Join(logsdir, fmt.Sprintf("%d_%s_%s-%s.log", r.FailureCount, pod.Namespace, pod.Name, container.Name)), os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-			if err != nil {
-				fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-				return
-			}
-			defer current.Close()
-
-			previous, err := os.OpenFile(filepath.Join(logsdir, fmt.Sprintf("%d_%s_%s-%s_previous.log", r.FailureCount, pod.Namespace, pod.Name, container.Name)), os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-			if err != nil {
-				fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-				return
-			}
-			defer previous.Close()
-
-			logStart := metav1.NewTime(startTime)
-			logs, err := kubeCli.CoreV1().Pods(pod.Namespace).GetLogs(pod.Name, &v1.PodLogOptions{SinceTime: &logStart, Container: container.Name}).DoRaw(context.TODO())
-			if err == nil {
-				fmt.Fprintln(current, string(logs))
-			}
-
-			logs, err = kubeCli.CoreV1().Pods(pod.Namespace).GetLogs(pod.Name, &v1.PodLogOptions{SinceTime: &logStart, Container: container.Name, Previous: true}).DoRaw(context.TODO())
-			if err == nil {
-				fmt.Fprintln(previous, string(logs))
-			}
-		}
-	}
-}
-
-func (r *KubernetesReporter) logEvents(kubeCli *kubernetes.Clientset, since time.Duration) {
-	f, err := os.OpenFile(filepath.Join(r.artifactsDir, fmt.Sprintf("%d_events.log", r.FailureCount)),
-		os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "failed to open the file: %v\n", err)
-		return
-	}
-	defer f.Close()
-
-	startTime := time.Now().Add(-since).Add(-5 * time.Second)
-
-	events, err := kubeCli.CoreV1().Events(v1.NamespaceAll).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		return
-	}
-
-	e := events.Items
-	sort.Slice(e, func(i, j int) bool {
-		return e[i].LastTimestamp.After(e[j].LastTimestamp.Time)
-	})
-
-	eventsToPrint := v1.EventList{}
-	for _, event := range e {
-		if event.LastTimestamp.Time.After(startTime) {
-			eventsToPrint.Items = append(eventsToPrint.Items, event)
-		}
-	}
-
-	j, err := json.MarshalIndent(eventsToPrint, "", "    ")
-	if err != nil {
-		return
-	}
-	fmt.Fprintln(f, string(j))
-}
