@@ -39,6 +39,10 @@ func (f *Framework) CreateBoundPVCFromDefinition(def *k8sv1.PersistentVolumeClai
 	gomega.Expect(err).ToNot(gomega.HaveOccurred())
 
 	f.ForceBindIfWaitForFirstConsumer(pvc)
+	err = utils.WaitForPersistentVolumeClaimPhase(f.K8sClient, pvc.Namespace, k8sv1.ClaimBound, pvc.Name)
+	gomega.Expect(err).ToNot(gomega.HaveOccurred())
+	pvc, err = utils.FindPVC(f.K8sClient, pvc.Namespace, pvc.Name)
+	gomega.Expect(err).ToNot(gomega.HaveOccurred())
 	return pvc
 }
 
@@ -234,7 +238,7 @@ func (f *Framework) GetMD5(namespace *k8sv1.Namespace, pvc *k8sv1.PersistentVolu
 	}
 
 	var output, stderr string
-	err = wait.PollImmediate(2*time.Second, 10*time.Second, func() (bool, error) {
+	err = wait.PollUntilContextTimeout(context.TODO(), 2*time.Second, 10*time.Second, true, func(_ context.Context) (bool, error) {
 		output, stderr, err = f.ExecShellInPod(executorPod.Name, namespace.Name, cmd)
 		if err != nil {
 			fmt.Fprintf(ginkgo.GinkgoWriter, "INFO: pod command execution failed, retrying: stderr: [%s]\n", stderr)
