@@ -27,6 +27,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
+	"k8s.io/utils/ptr"
 
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
@@ -47,7 +48,7 @@ const (
 	BlockDeviceMD5                   = "7c55761d39e6428fa27c21d8710a3d19"
 )
 
-var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:component]Importer Test Suite", func() {
+var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:component]Importer Test Suite", Serial, func() {
 	var (
 		ns string
 		f  = framework.NewFramework(namespacePrefix)
@@ -93,7 +94,7 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 		// deleting the PVC at the end of the test, so if another runs first we will fail.
 		pvc, err := f.CreatePVCFromDefinition(utils.NewPVCDefinition("no-import-ann", "1G", nil, nil))
 		By("Verifying PVC with no annotation remains empty")
-		matchString := "PVC annotation not found, skipping pvc\t{\"PVC\": \"" + ns + "/" + pvc.Name + "\", \"annotation\": \"" + controller.AnnEndpoint + "\"}"
+		matchString := fmt.Sprintf("PVC annotation not found, skipping pvc\t{\"PVC\": {\"name\":\"%s\",\"namespace\":\"%s\"}, \"annotation\": \"%s\"}", pvc.Name, ns, controller.AnnEndpoint)
 		fmt.Fprintf(GinkgoWriter, "INFO: matchString: [%s]\n", matchString)
 		Eventually(func() string {
 			log, err := f.RunKubectlCommand("logs", f.ControllerPod.Name, "-n", f.CdiInstallNs)
@@ -189,7 +190,7 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 	})
 })
 
-var _ = Describe("DataVolume Garbage Collection", func() {
+var _ = Describe("DataVolume Garbage Collection", Serial, func() {
 	var (
 		f        = framework.NewFramework(namespacePrefix)
 		ns       string
@@ -278,7 +279,7 @@ var _ = Describe("DataVolume Garbage Collection", func() {
 	)
 })
 
-var _ = Describe("[Istio] Namespace sidecar injection", func() {
+var _ = Describe("[Istio] Namespace sidecar injection", Serial, func() {
 	var (
 		f = framework.NewFramework(namespacePrefix)
 
@@ -325,7 +326,9 @@ var _ = Describe("[Istio] Namespace sidecar injection", func() {
 		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
 		// We set the Immediate Binding annotation to true, to eliminate creation of the consumer pod, which will also fail due to the Istio sidecar.
 		dataVolume.Annotations[controller.AnnImmediateBinding] = "true"
-		dataVolume.Annotations[controller.AnnPodSidecarInjection] = "true"
+		// A single service mesh provider is deployed so either Istio or Linkerd, not both.
+		dataVolume.Annotations[controller.AnnPodSidecarInjectionIstio] = "true"
+		dataVolume.Annotations[controller.AnnPodSidecarInjectionLinkerd] = "true"
 		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -372,7 +375,7 @@ var _ = Describe("[Istio] Namespace sidecar injection", func() {
 	})
 })
 
-var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", func() {
+var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", Serial, func() {
 	var cr *cdiv1.CDI
 	var oldSpec *cdiv1.CDISpec
 	f := framework.NewFramework(namespacePrefix)
@@ -435,12 +438,15 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 		Expect(err).NotTo(HaveOccurred(), "Unable to get importer pod")
 
 		By("Verify the import pod has nodeSelector")
-		match := f.PodSpecHasTestNodePlacementValues(importer.Spec)
-		Expect(match).To(BeTrue(), fmt.Sprintf("node placement in pod spec\n%v\n differs from node placement values in CDI CR\n%v\n", importer.Spec, cr.Spec.Workloads))
+		Expect(importer.Spec.NodeSelector).To(Equal(framework.NodeSelectorTestValue))
+		By("Verify the import pod has affinity")
+		Expect(importer.Spec.Affinity).To(Equal(framework.AffinityTestValue))
+		By("Verify the import pod has tolerations")
+		Expect(importer.Spec.Tolerations).To(ContainElement(framework.TolerationsTestValue[0]))
 	})
 })
 
-var _ = Describe("[rfe_id:1118][crit:high][vendor:cnv-qe@redhat.com][level:component]Importer Test Suite-prometheus", func() {
+var _ = Describe("[rfe_id:1118][crit:high][vendor:cnv-qe@redhat.com][level:component]Importer Test Suite-prometheus", Serial, func() {
 	var prometheusURL string
 	var portForwardCmd *exec.Cmd
 	client := &http.Client{
@@ -668,7 +674,7 @@ var _ = Describe("Importer Test Suite-Block_device", func() {
 	})
 })
 
-var _ = Describe("[rfe_id:1947][crit:high][test_id:2145][vendor:cnv-qe@redhat.com][level:component]Importer Archive ContentType", func() {
+var _ = Describe("[rfe_id:1947][crit:high][test_id:2145][vendor:cnv-qe@redhat.com][level:component]Importer Archive ContentType", Serial, func() {
 	f := framework.NewFramework(namespacePrefix)
 
 	It("Should import archive content type tar file", func() {
@@ -694,7 +700,7 @@ var _ = Describe("[rfe_id:1947][crit:high][test_id:2145][vendor:cnv-qe@redhat.co
 	})
 })
 
-var _ = Describe("PVC import phase matches pod phase", func() {
+var _ = Describe("PVC import phase matches pod phase", Serial, func() {
 	f := framework.NewFramework(namespacePrefix)
 
 	It("[test_id:4980]Should never go to failed even if import fails", func() {
@@ -725,7 +731,7 @@ var _ = Describe("PVC import phase matches pod phase", func() {
 	})
 })
 
-var _ = Describe("Namespace with quota", func() {
+var _ = Describe("Namespace with quota", Serial, func() {
 	f := framework.NewFramework(namespacePrefix)
 	var (
 		orgConfig *v1.ResourceRequirements
@@ -948,6 +954,10 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 	})
 
 	It("[test_id:3996] Import datavolume with bad url will increase dv retry count", func() {
+		if f.IsPrometheusAvailable() {
+			dataVolumeNoUnusualRestartTest(f)
+		}
+
 		dvName := "import-dv-bad-url"
 		By(fmt.Sprintf("Creating new datavolume %s", dvName))
 		dv := utils.NewDataVolumeWithHTTPImport(dvName, "100Mi", invalidQcowImagesURL())
@@ -1302,7 +1312,7 @@ var _ = Describe("Preallocation", func() {
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreQcow2URL(), f.BlockSCName)
 		}),
-		Entry("ImageIO import", true, utils.ImageioMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
+		Entry("ImageIO import", Serial, true, utils.ImageioMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			cm, err := utils.CopyImageIOCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
 			Expect(err).ToNot(HaveOccurred())
 			stringData := map[string]string{
@@ -1384,7 +1394,7 @@ var _ = Describe("Preallocation", func() {
 		}),
 	)
 
-	It("Filesystem overhead is honored with blank volume", func() {
+	It("Filesystem overhead is honored with blank volume", Serial, func() {
 		tests.SetFilesystemOverhead(f, "0.055", "0.055")
 
 		dv := utils.NewDataVolumeForBlankRawImage("import-dv", "100Mi")
@@ -1610,10 +1620,20 @@ var _ = Describe("Import populator", func() {
 		By("Delete import population PVC")
 		err = f.DeletePVC(pvc)
 		Expect(err).ToNot(HaveOccurred())
+
+		tests.DisableWebhookPvcRendering(f.CrClient)
 	})
 
-	DescribeTable("should import fileSystem PVC", func(expectedMD5 string, volumeImportSourceFunc func(cdiv1.DataVolumeContentType, bool) error, preallocation bool) {
+	DescribeTable("should import fileSystem PVC", func(expectedMD5 string, volumeImportSourceFunc func(cdiv1.DataVolumeContentType, bool) error, preallocation, webhookRendering bool) {
 		pvc = importPopulationPVCDefinition()
+
+		if webhookRendering {
+			tests.EnableWebhookPvcRendering(f.CrClient)
+			controller.AddLabel(pvc, common.PvcApplyStorageProfileLabel, "true")
+			// Unset AccessModes which will be set by the webhook rendering
+			pvc.Spec.AccessModes = nil
+		}
+
 		pvc = f.CreateScheduledPVCFromDefinition(pvc)
 		err = volumeImportSourceFunc(cdiv1.DataVolumeKubeVirt, preallocation)
 		Expect(err).ToNot(HaveOccurred())
@@ -1663,16 +1683,17 @@ var _ = Describe("Import populator", func() {
 			return err != nil && k8serrors.IsNotFound(err)
 		}, timeout, pollingInterval).Should(BeTrue())
 	},
-		Entry("with HTTP image and preallocation", utils.TinyCoreMD5, createHTTPImportPopulatorCR, true),
-		Entry("with HTTP image without preallocation", utils.TinyCoreMD5, createHTTPImportPopulatorCR, false),
-		Entry("with Registry image and preallocation", utils.TinyCoreMD5, createRegistryImportPopulatorCR, true),
-		Entry("with Registry image without preallocation", utils.TinyCoreMD5, createRegistryImportPopulatorCR, false),
-		Entry("with ImageIO image with preallocation", utils.ImageioMD5, createImageIOImportPopulatorCR, true),
-		Entry("with ImageIO image without preallocation", utils.ImageioMD5, createImageIOImportPopulatorCR, false),
-		Entry("with VDDK image with preallocation", utils.VcenterMD5, createVDDKImportPopulatorCR, true),
-		Entry("with VDDK image without preallocation", utils.VcenterMD5, createVDDKImportPopulatorCR, false),
-		Entry("with Blank image with preallocation", utils.BlankMD5, createBlankImportPopulatorCR, true),
-		Entry("with Blank image without preallocation", utils.BlankMD5, createBlankImportPopulatorCR, false),
+		Entry("[test_id:11001]with HTTP image and preallocation", utils.TinyCoreMD5, createHTTPImportPopulatorCR, true, false),
+		Entry("[test_id:11002]with HTTP image without preallocation", utils.TinyCoreMD5, createHTTPImportPopulatorCR, false, false),
+		Entry("[rfe_id:10985][crit:high][test_id:11003]with HTTP image and preallocation, with incomplete PVC webhook rendering", Serial, utils.TinyCoreMD5, createHTTPImportPopulatorCR, true, true),
+		Entry("[test_id:11004]with Registry image and preallocation", utils.TinyCoreMD5, createRegistryImportPopulatorCR, true, false),
+		Entry("[test_id:11005]with Registry image without preallocation", utils.TinyCoreMD5, createRegistryImportPopulatorCR, false, false),
+		Entry("[test_id:11006]with ImageIO image with preallocation", Serial, utils.ImageioMD5, createImageIOImportPopulatorCR, true, false),
+		Entry("[test_id:11007]with ImageIO image without preallocation", Serial, utils.ImageioMD5, createImageIOImportPopulatorCR, false, false),
+		Entry("[test_id:11008]with VDDK image with preallocation", utils.VcenterMD5, createVDDKImportPopulatorCR, true, false),
+		Entry("[test_id:11009]with VDDK image without preallocation", utils.VcenterMD5, createVDDKImportPopulatorCR, false, false),
+		Entry("[test_id:11010]with Blank image with preallocation", utils.BlankMD5, createBlankImportPopulatorCR, true, false),
+		Entry("[test_id:11011]with Blank image without preallocation", utils.BlankMD5, createBlankImportPopulatorCR, false, false),
 	)
 
 	DescribeTable("should import Block PVC", func(expectedMD5 string, volumeImportSourceFunc func(cdiv1.DataVolumeContentType, bool) error) {
@@ -1717,7 +1738,7 @@ var _ = Describe("Import populator", func() {
 	},
 		Entry("with HTTP image", utils.TinyCoreMD5, createHTTPImportPopulatorCR),
 		Entry("with Registry image", utils.TinyCoreMD5, createRegistryImportPopulatorCR),
-		Entry("with ImageIO image", utils.ImageioMD5, createImageIOImportPopulatorCR),
+		Entry("with ImageIO image", Serial, utils.ImageioMD5, createImageIOImportPopulatorCR),
 		Entry("with VDDK image", utils.VcenterMD5, createVDDKImportPopulatorCR),
 		Entry("with Blank image", utils.BlankMD5, createBlankImportPopulatorCR),
 	)
@@ -2078,6 +2099,104 @@ var _ = Describe("Import populator", func() {
 		err = f.DeletePVC(pvcPrime)
 		Expect(err).ToNot(HaveOccurred())
 	})
+
+	It("should recreate and reimport pvc if it was deleted", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs))
+		dv, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		pvc, err = utils.WaitForPVC(f.K8sClient, dv.Namespace, dv.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		By("Wait for import to be completed")
+		err = utils.WaitForDataVolumePhase(f, dv.Namespace, cdiv1.Succeeded, dv.Name)
+		Expect(err).ToNot(HaveOccurred(), "Datavolume not in phase succeeded in time")
+
+		By("Delete PVC and wait for it to be deleted")
+		err = f.DeletePVC(pvc)
+		Expect(err).ToNot(HaveOccurred())
+		deleted, err := f.WaitPVCDeletedByUID(pvc, time.Minute)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(deleted).To(BeTrue())
+
+		pvc, err = utils.WaitForPVC(f.K8sClient, dv.Namespace, dv.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		By("Verify target PVC is bound again")
+		err = utils.WaitForPersistentVolumeClaimPhase(f.K8sClient, pvc.Namespace, v1.ClaimBound, pvc.Name)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verify content")
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.MD5PrefixSize)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(md5).To(Equal(utils.TinyCoreMD5))
+	})
+})
+
+var _ = Describe("Containerdisk envs to PVC labels", func() {
+	f := framework.NewFramework(namespacePrefix)
+
+	// The corresponding env var is defined in tests/BUILD.bazel (for pullMethod node)
+	// and tools/cdi-func-test-registry-init/populate-registry.sh (for pullMethod pod).
+	const (
+		testKubevirtIoKey           = "test.kubevirt.io/test"
+		testKubevirtIoValue         = "testvalue"
+		testKubevirtIoKeyExisting   = "test.kubevirt.io/existing"
+		testKubevirtIoValueExisting = "existing"
+	)
+
+	var (
+		tinyCoreRegistryURL = func() string { return fmt.Sprintf(utils.TinyCoreIsoRegistryURL, f.CdiInstallNs) }
+		trustedRegistryURL  = func() string { return fmt.Sprintf(utils.TrustedRegistryURL, f.DockerPrefix) }
+		trustedRegistryIS   = func() string { return fmt.Sprintf(utils.TrustedRegistryIS, f.DockerPrefix) }
+	)
+
+	DescribeTable("Import should add KUBEVIRT_IO_ env vars to PVC labels when source is registry", func(pullMethod cdiv1.RegistryPullMethod, urlFn func() string, isImageStream bool) {
+		dataVolume := utils.NewDataVolumeWithRegistryImport("import-dv", "100Mi", urlFn())
+		// The existing key should not be overwritten
+		dataVolume.ObjectMeta.Labels = map[string]string{
+			testKubevirtIoKeyExisting: testKubevirtIoValueExisting,
+		}
+
+		if isImageStream {
+			dataVolume.Spec.Source.Registry.URL = nil
+			dataVolume.Spec.Source.Registry.ImageStream = ptr.To(urlFn())
+			dataVolume.Annotations[controller.AnnPodRetainAfterCompletion] = "true"
+		}
+
+		dataVolume.Spec.Source.Registry.PullMethod = &pullMethod
+		if pullMethod == cdiv1.RegistryPullPod {
+			cm, err := utils.CopyRegistryCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
+			Expect(err).ToNot(HaveOccurred())
+			dataVolume.Spec.Source.Registry.CertConfigMap = &cm
+		}
+
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		pvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		phase := cdiv1.Succeeded
+		By(fmt.Sprintf("Waiting for datavolume to match phase %s", string(phase)))
+		err = utils.WaitForDataVolumePhase(f, f.Namespace.Name, phase, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+
+		pvc, err = utils.FindPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+
+		Eventually(func(g Gomega) {
+			g.Expect(pvc.GetLabels()).To(HaveKeyWithValue(testKubevirtIoKey, testKubevirtIoValue))
+			g.Expect(pvc.GetLabels()).To(HaveKeyWithValue(testKubevirtIoKeyExisting, testKubevirtIoValueExisting))
+		}, timeout, pollingInterval).Should(Succeed())
+	},
+		Entry("with pullMethod pod", cdiv1.RegistryPullPod, tinyCoreRegistryURL, false),
+		Entry("with pullMethod node", cdiv1.RegistryPullNode, trustedRegistryURL, false),
+		Entry("with pullMethod node", cdiv1.RegistryPullNode, trustedRegistryIS, true),
+	)
 })
 
 func generateRegistryOnlySidecar() *unstructured.Unstructured {
