@@ -6,37 +6,37 @@ import (
 	"fmt"
 	"reflect"
 	"regexp"
+	"strings"
 	"time"
 
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-
 	routev1 "github.com/openshift/api/route/v1"
 	routeclient "github.com/openshift/client-go/route/clientset/versioned"
-	secclient "github.com/openshift/client-go/security/clientset/versioned"
-	conditions "github.com/openshift/custom-resource-status/conditions/v1"
 	appsv1 "k8s.io/api/apps/v1"
-	corev1 "k8s.io/api/core/v1"
-	rbacv1 "k8s.io/api/rbac/v1"
 	schedulev1 "k8s.io/api/scheduling/v1"
-	"k8s.io/apimachinery/pkg/api/errors"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
-	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/utils/pointer"
+	"kubevirt.io/containerized-data-importer/pkg/controller"
+	resourcesutils "kubevirt.io/containerized-data-importer/pkg/operator/resources/utils"
+	"kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk"
 	crclient "sigs.k8s.io/controller-runtime/pkg/client"
 
+	secclient "github.com/openshift/client-go/security/clientset/versioned"
+	conditions "github.com/openshift/custom-resource-status/conditions/v1"
+	corev1 "k8s.io/api/core/v1"
+	rbacv1 "k8s.io/api/rbac/v1"
+	"k8s.io/apimachinery/pkg/api/errors"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/utils/ptr"
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
-	"kubevirt.io/containerized-data-importer/pkg/controller"
 	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
-	resourcesutils "kubevirt.io/containerized-data-importer/pkg/operator/resources/utils"
 	"kubevirt.io/containerized-data-importer/tests/framework"
 	"kubevirt.io/containerized-data-importer/tests/utils"
 	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/api"
-	"kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk"
 )
 
 var (
@@ -44,7 +44,7 @@ var (
 )
 
 var _ = Describe("ALL Operator tests", func() {
-	Context("[Destructive]", func() {
+	Context("[Destructive]", Serial, func() {
 		var _ = Describe("Operator tests", func() {
 			f := framework.NewFramework("operator-test")
 
@@ -328,7 +328,7 @@ var _ = Describe("ALL Operator tests", func() {
 					Operator: corev1.TolerationOpExists,
 				}
 
-				if !tolerationExists(cr.Spec.Infra.Tolerations, criticalAddonsToleration) {
+				if !tolerationExists(cr.Spec.Infra.NodePlacement.Tolerations, criticalAddonsToleration) {
 					Skip("Unexpected CDI CR (not from cdi-cr.yaml), doesn't tolerate CriticalAddonsOnly")
 				}
 
@@ -626,7 +626,9 @@ var _ = Describe("ALL Operator tests", func() {
 			It("[test_id:4782] Should install CDI infrastructure pods with node placement", func() {
 				By("Creating modified CDI CR, with infra nodePlacement")
 				localSpec := restoreCdiCr.Spec.DeepCopy()
-				localSpec.Infra = f.TestNodePlacementValues()
+				nodePlacement := f.TestNodePlacementValues()
+
+				localSpec.Infra.NodePlacement = nodePlacement
 
 				tempCdiCr := &cdiv1.CDI{
 					ObjectMeta: metav1.ObjectMeta{
@@ -641,9 +643,14 @@ var _ = Describe("ALL Operator tests", func() {
 				for _, deploymentName := range []string{"cdi-apiserver", "cdi-deployment", "cdi-uploadproxy"} {
 					deployment, err := f.K8sClient.AppsV1().Deployments(f.CdiInstallNs).Get(context.TODO(), deploymentName, metav1.GetOptions{})
 					Expect(err).ToNot(HaveOccurred())
+					By("Verify the deployment has nodeSelector")
+					Expect(deployment.Spec.Template.Spec.NodeSelector).To(Equal(framework.NodeSelectorTestValue))
 
-					match := f.PodSpecHasTestNodePlacementValues(deployment.Spec.Template.Spec)
-					Expect(match).To(BeTrue(), fmt.Sprintf("node placement in pod spec\n%v\n differs from node placement values in CDI CR\n%v\n", deployment.Spec.Template.Spec, localSpec.Infra))
+					By("Verify the deployment has affinity")
+					checkAntiAffinity(deploymentName, deployment.Spec.Template.Spec.Affinity)
+
+					By("Verify the deployment has tolerations")
+					Expect(deployment.Spec.Template.Spec.Tolerations).To(ContainElement(framework.TolerationsTestValue[0]))
 				}
 			})
 		})
@@ -794,7 +801,7 @@ var _ = Describe("ALL Operator tests", func() {
 				Expect(err).ToNot(HaveOccurred())
 
 				By("Overwrite priority of SCC")
-				scc.Priority = pointer.Int32(10)
+				scc.Priority = ptr.To[int32](10)
 				_, err = secClient.SecurityV1().SecurityContextConstraints().Update(context.TODO(), scc, metav1.UpdateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
@@ -804,6 +811,161 @@ var _ = Describe("ALL Operator tests", func() {
 					return scc.Priority
 				}, 2*time.Minute, 1*time.Second).Should(BeNil())
 			})
+			It("[test_id:4785] Should update infra pod number when modify the replica in CDI CR", func() {
+				By("Modify the replica separately")
+				cdi := getCDI(f)
+				apiserverTmpReplica := int32(2)
+				deploymentTmpReplica := int32(3)
+				uploadproxyTmpReplica := int32(4)
+
+				cdi.Spec.Infra.APIServerReplicas = &apiserverTmpReplica
+				cdi.Spec.Infra.DeploymentReplicas = &deploymentTmpReplica
+				cdi.Spec.Infra.UploadProxyReplicas = &uploadproxyTmpReplica
+
+				_, err := f.CdiClient.CdiV1beta1().CDIs().Update(context.TODO(), cdi, metav1.UpdateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				Eventually(func() bool {
+					for _, deploymentName := range []string{"cdi-apiserver", "cdi-deployment", "cdi-uploadproxy"} {
+						depl, err := f.K8sClient.AppsV1().Deployments(f.CdiInstallNs).Get(context.TODO(), deploymentName, metav1.GetOptions{})
+						Expect(err).ToNot(HaveOccurred())
+						if err != nil || *depl.Spec.Replicas == 1 {
+							return false
+						}
+					}
+					By("Replicas in deployments update complete")
+					return true
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+				By("Verify the replica of cdi-apiserver")
+
+				Eventually(func() bool {
+					return getPodNumByPrefix(f, "cdi-apiserver") == 2
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+				By("Verify the replica of cdi-deployment")
+				Eventually(func() bool {
+					return getPodNumByPrefix(f, "cdi-deployment") == 3
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+				By("Verify the replica of cdi-uploadproxy")
+				Eventually(func() bool {
+					return getPodNumByPrefix(f, "cdi-uploadproxy") == 4
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+				By("Reset replica for CDI CR")
+				cdi = getCDI(f)
+				cdi.Spec.Infra.APIServerReplicas = nil
+				cdi.Spec.Infra.DeploymentReplicas = nil
+				cdi.Spec.Infra.UploadProxyReplicas = nil
+
+				_, err = f.CdiClient.CdiV1beta1().CDIs().Update(context.TODO(), cdi, metav1.UpdateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				By("Replica should be 1 when replica dosen't set in CDI CR")
+
+				Eventually(func() bool {
+					for _, deploymentName := range []string{"cdi-apiserver", "cdi-deployment", "cdi-uploadproxy"} {
+						depl, err := f.K8sClient.AppsV1().Deployments(f.CdiInstallNs).Get(context.TODO(), deploymentName, metav1.GetOptions{})
+						Expect(err).ToNot(HaveOccurred())
+						_, err = utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, deploymentName, common.CDIComponentLabel+"="+deploymentName)
+						if err != nil || *depl.Spec.Replicas != 1 {
+							return false
+						}
+
+					}
+					return true
+
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+			})
+			It("Should update infra deployments when modify customizeComponents in CDI Cr", func() {
+				By("Modify the customizeComponents separately")
+				cdi := getCDI(f)
+				testJsonPatch := "test-json-patch"
+				testStrategicPatch := "test-strategic-patch"
+				testMergePatch := "test-merge-patch"
+				cdi.Spec.CustomizeComponents = cdiv1.CustomizeComponents{
+					Patches: []cdiv1.CustomizeComponentsPatch{
+						{
+							ResourceName: "cdi-apiserver",
+							ResourceType: "Deployment",
+							Patch:        fmt.Sprintf(`[{"op":"add","path":"/metadata/annotations/%s","value":"%s"}]`, testJsonPatch, testJsonPatch),
+							Type:         cdiv1.JSONPatchType,
+						},
+						{
+							ResourceName: "cdi-deployment",
+							ResourceType: "Deployment",
+							Patch:        fmt.Sprintf(`{"metadata": {"annotations": {"%s": "%s"}}}`, testStrategicPatch, testStrategicPatch),
+							Type:         cdiv1.StrategicMergePatchType,
+						},
+						{
+							ResourceName: "cdi-uploadproxy",
+							ResourceType: "Deployment",
+							Patch:        fmt.Sprintf(`{"metadata": {"annotations": {"%s": "%s"}}}`, testMergePatch, testMergePatch),
+							Type:         cdiv1.MergePatchType,
+						},
+					},
+					Flags: &cdiv1.Flags{
+						API:         map[string]string{"v": "5", "skip_headers": ""},
+						Controller:  map[string]string{"v": "6", "skip_headers": ""},
+						UploadProxy: map[string]string{"v": "7", "skip_headers": ""},
+					},
+				}
+				_, err := f.CdiClient.CdiV1beta1().CDIs().Update(context.TODO(), cdi, metav1.UpdateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				Eventually(func() bool {
+					for _, deploymentName := range []string{"cdi-apiserver", "cdi-deployment", "cdi-uploadproxy"} {
+						depl, err := f.K8sClient.AppsV1().Deployments(f.CdiInstallNs).Get(context.TODO(), deploymentName, metav1.GetOptions{})
+						Expect(err).ToNot(HaveOccurred())
+
+						if err != nil || depl.GetAnnotations()[cc.AnnCdiCustomizeComponentHash] == "" {
+							return false
+						}
+					}
+					By("Patches applied")
+					return true
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+				verifyPatches := func(deployment, annoKey, annoValue string, desiredArgs ...string) {
+					By(fmt.Sprintf("Verify patches of %s", deployment))
+					Eventually(func() bool {
+						depl, err := f.K8sClient.AppsV1().Deployments(f.CdiInstallNs).Get(context.TODO(), deployment, metav1.GetOptions{})
+						Expect(err).ToNot(HaveOccurred())
+						args := strings.Join(depl.Spec.Template.Spec.Containers[0].Args, " ")
+						for _, a := range desiredArgs {
+							if !strings.Contains(args, a) {
+								return false
+							}
+						}
+						return depl.GetAnnotations()[annoKey] == annoValue
+					}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+				}
+				verifyPatches("cdi-apiserver", testJsonPatch, testJsonPatch, "-v 5", "-skip_headers")
+				verifyPatches("cdi-deployment", testStrategicPatch, testStrategicPatch, "-v 6", "-skip_headers")
+				verifyPatches("cdi-uploadproxy", testMergePatch, testMergePatch, "-v 7", "-skip_headers")
+
+				By("Reset CustomizeComponents for CDI CR")
+				cdi = getCDI(f)
+
+				cdi.Spec.CustomizeComponents = cdiv1.CustomizeComponents{}
+				_, err = f.CdiClient.CdiV1beta1().CDIs().Update(context.TODO(), cdi, metav1.UpdateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				Eventually(func() bool {
+					for _, deploymentName := range []string{"cdi-apiserver", "cdi-deployment", "cdi-uploadproxy"} {
+						depl, err := f.K8sClient.AppsV1().Deployments(f.CdiInstallNs).Get(context.TODO(), deploymentName, metav1.GetOptions{})
+						Expect(err).ToNot(HaveOccurred())
+
+						_, err = utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, deploymentName, common.CDIComponentLabel+"="+deploymentName)
+						if err != nil || depl.GetAnnotations()[cc.AnnCdiCustomizeComponentHash] != "" {
+							return false
+						}
+					}
+					return true
+				}, 5*time.Minute, 1*time.Second).Should(BeTrue())
+
+			})
 		})
 
 		var _ = Describe("Operator cert config tests", func() {
@@ -999,9 +1161,9 @@ var _ = Describe("ALL Operator tests", func() {
 				// Deployment
 				verifyPodPriorityClass(cdiDeploymentPodPrefix, string(prioClass), common.CDILabelSelector)
 				// API server
-				verifyPodPriorityClass(cdiApiServerPodPrefix, string(prioClass), "")
+				verifyPodPriorityClass(cdiApiServerPodPrefix, string(prioClass), common.CDILabelSelector)
 				// Upload server
-				verifyPodPriorityClass(cdiUploadProxyPodPrefix, string(prioClass), "")
+				verifyPodPriorityClass(cdiUploadProxyPodPrefix, string(prioClass), common.CDILabelSelector)
 				By("Verifying there is just a single cdi controller pod")
 				Eventually(func() error {
 					_, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, cdiDeploymentPodPrefix, common.CDILabelSelector)
@@ -1027,9 +1189,9 @@ var _ = Describe("ALL Operator tests", func() {
 				By("Verifying the CDI deployment is updated")
 				verifyPodPriorityClass(cdiDeploymentPodPrefix, string(systemClusterCritical), common.CDILabelSelector)
 				By("Verifying the CDI api server is updated")
-				verifyPodPriorityClass(cdiApiServerPodPrefix, string(systemClusterCritical), "")
+				verifyPodPriorityClass(cdiApiServerPodPrefix, string(systemClusterCritical), common.CDILabelSelector)
 				By("Verifying the CDI upload proxy server is updated")
-				verifyPodPriorityClass(cdiUploadProxyPodPrefix, string(systemClusterCritical), "")
+				verifyPodPriorityClass(cdiUploadProxyPodPrefix, string(systemClusterCritical), common.CDILabelSelector)
 			})
 
 			It("should use openshift priority class if not set and available", func() {
@@ -1043,9 +1205,9 @@ var _ = Describe("ALL Operator tests", func() {
 				// Deployment
 				verifyPodPriorityClass(cdiDeploymentPodPrefix, string(osUserCrit.Name), common.CDILabelSelector)
 				// API server
-				verifyPodPriorityClass(cdiApiServerPodPrefix, string(osUserCrit.Name), "")
+				verifyPodPriorityClass(cdiApiServerPodPrefix, string(osUserCrit.Name), common.CDILabelSelector)
 				// Upload server
-				verifyPodPriorityClass(cdiUploadProxyPodPrefix, string(osUserCrit.Name), "")
+				verifyPodPriorityClass(cdiUploadProxyPodPrefix, string(osUserCrit.Name), common.CDILabelSelector)
 			})
 		})
 	})
@@ -1260,8 +1422,44 @@ func checkLogForRegEx(regEx *regexp.Regexp, log string) bool {
 	return len(matches) >= 1
 }
 
+func checkAntiAffinity(name string, deploymentAffinity *corev1.Affinity) {
+
+	affinityTampleValue := &corev1.PodAntiAffinity{
+		PreferredDuringSchedulingIgnoredDuringExecution: []corev1.WeightedPodAffinityTerm{
+			{
+				Weight: int32(1),
+				PodAffinityTerm: corev1.PodAffinityTerm{
+					LabelSelector: &metav1.LabelSelector{
+						MatchExpressions: []metav1.LabelSelectorRequirement{
+							{
+								Key:      "cdi.kubevirt.io",
+								Operator: metav1.LabelSelectorOpIn,
+								Values:   []string{name}},
+						},
+					},
+					TopologyKey: "kubernetes.io/hostname",
+				},
+			},
+		},
+	}
+	affCopy := framework.AffinityTestValue.DeepCopy()
+	affCopy.PodAntiAffinity = affinityTampleValue
+	Expect(reflect.DeepEqual(deploymentAffinity, affCopy)).To(BeTrue())
+
+}
+
 func getLog(f *framework.Framework, name string) string {
 	log, err := f.RunKubectlCommand("logs", "--since=0", name, "-n", f.CdiInstallNs)
 	Expect(err).ToNot(HaveOccurred())
 	return log
 }
+func getPodNumByPrefix(f *framework.Framework, deploymentName string) int {
+	labelSelector := metav1.LabelSelector{MatchLabels: map[string]string{common.CDIComponentLabel: deploymentName}}
+
+	podList, err := f.K8sClient.CoreV1().Pods(f.CdiInstallNs).List(context.TODO(), metav1.ListOptions{
+		LabelSelector: labels.Set(labelSelector.MatchLabels).String(),
+	})
+	Expect(err).ToNot(HaveOccurred(), "failed listing deployment pods")
+
+	return len(podList.Items)
+}
