@@ -244,10 +244,42 @@ var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:compon
 			By("Try upload again")
 			err = uploadImage(uploadProxyURL, token, http.StatusServiceUnavailable)
 			Expect(err).ToNot(HaveOccurred())
-
 		})
 
-		DescribeTable("Verify validation error message on async upload if virtual size > pvc size", func(filename string) {
+		It("Verify cross-site scripting XSS attempt is escaped accordingly to avoid attack", func() {
+			By("Verify PVC annotation says ready")
+			const XSSAttempt = "<script>Bad stuff here...</script>"
+			found, err := utils.WaitPVCPodStatusReady(f.K8sClient, pvc)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(found).To(BeTrue())
+			pvc, err = f.K8sClient.CoreV1().PersistentVolumeClaims(pvc.Namespace).Get(context.TODO(), pvc.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			pvc.Annotations[controller.AnnContentType] = XSSAttempt
+			pvc, err = f.K8sClient.CoreV1().PersistentVolumeClaims(pvc.Namespace).Update(context.TODO(), pvc, metav1.UpdateOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			var token string
+			By("Get an upload token")
+			token, err = utils.RequestUploadToken(f.CdiClient, pvc)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(token).ToNot(BeEmpty())
+
+			By("Do upload")
+			resp, err := getUploadToPathResponse(binaryRequestFunc, utils.UploadFile, uploadProxyURL, syncUploadPath, token)
+			Expect(err).ToNot(HaveOccurred())
+			body, err := io.ReadAll(resp.Body)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Verify XSS attempt")
+			// Verify XSS attack is not present in the error msg
+			Expect(string(body)).ToNot(ContainSubstring(XSSAttempt))
+			// Verify < and > characters are escaped accordingly
+			Expect(string(body)).To(ContainSubstring("&lt;script&gt;"))
+			// Verify PVC name is intact
+			Expect(string(body)).To(ContainSubstring(pvc.Name))
+		})
+
+		DescribeTable("Verify validation error message on async upload if virtual size > pvc size", Serial, func(filename string) {
 			By("Verify PVC annotation says ready")
 			found, err := utils.WaitPVCPodStatusReady(f.K8sClient, pvc)
 			Expect(err).ToNot(HaveOccurred())
@@ -268,7 +300,7 @@ var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:compon
 			Entry("fail given a large physical size QCOW2 file", utils.UploadFileLargePhysicalDiskQcow),
 		)
 
-		DescribeTable("[posneg:negative][test_id:2330]Verify failure on sync upload if virtual size > pvc size", func(filename string) {
+		DescribeTable("[posneg:negative][test_id:2330]Verify failure on sync upload if virtual size > pvc size", Serial, func(filename string) {
 			By("Verify PVC annotation says ready")
 			found, err := utils.WaitPVCPodStatusReady(f.K8sClient, pvc)
 			Expect(err).ToNot(HaveOccurred())
@@ -338,7 +370,7 @@ var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:compon
 			tinyCoreFileMd5, err := util.Md5sum(utils.UploadFile)
 			Expect(err).ToNot(HaveOccurred())
 			filesToUpload := map[string]string{utils.TinyCoreFile: tinyCoreFileMd5, utils.CirrosQCow2File: cirrosFileMd5}
-			archiveFilePath, err := utils.ArchiveFiles("archive", os.TempDir(), utils.UploadFile, utils.UploadCirrosFile)
+			archiveFilePath, err := utils.ArchiveFiles(fmt.Sprintf("archive-%s", f.Namespace.Name), os.TempDir(), utils.UploadFile, utils.UploadCirrosFile)
 			Expect(err).ToNot(HaveOccurred())
 			if format != "" {
 				archiveFilePath, err = utils.FormatTestData(archiveFilePath, os.TempDir(), format)
@@ -653,7 +685,7 @@ var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:compon
 				tinyCoreFileMd5, err := util.Md5sum(utils.UploadFile)
 				Expect(err).ToNot(HaveOccurred())
 				filesToUpload := map[string]string{utils.TinyCoreFile: tinyCoreFileMd5, utils.CirrosQCow2File: cirrosFileMd5}
-				archiveFilePath, err := utils.ArchiveFiles("archive", os.TempDir(), utils.UploadFile, utils.UploadCirrosFile)
+				archiveFilePath, err := utils.ArchiveFiles(fmt.Sprintf("archive-%s", f.Namespace.Name), os.TempDir(), utils.UploadFile, utils.UploadCirrosFile)
 				Expect(err).ToNot(HaveOccurred())
 				if format != "" {
 					archiveFilePath, err = utils.FormatTestData(archiveFilePath, os.TempDir(), format)
@@ -897,6 +929,32 @@ func uploadFileNameToPath(requestFunc uploadFileNameRequestCreator, fileName, po
 	return nil
 }
 
+func getUploadToPathResponse(requestFunc uploadFileNameRequestCreator, fileName, portForwardURL, path, token string) (*http.Response, error) {
+	url := portForwardURL + path
+
+	client := &http.Client{
+		Transport: &http.Transport{
+			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
+		},
+	}
+
+	req, err := requestFunc(url, fileName)
+	if err != nil {
+		return nil, err
+	}
+	defer req.Body.Close()
+
+	req.Header.Add("Authorization", "Bearer "+token)
+	req.Header.Add("Origin", "foo.bar.com")
+
+	resp, err := client.Do(req)
+	if err != nil {
+		return nil, err
+	}
+
+	return resp, nil
+}
+
 func uploadFileNameToPathWithClient(client *http.Client, requestFunc uploadFileNameRequestCreator, fileName, portForwardURL, path, token string, expectedStatus int) error {
 	url := portForwardURL + path
 
@@ -946,7 +1004,7 @@ func HasEnvironmentVariableFromSecret(pod *v1.Pod, name string, secret *v1.Secre
 	return false
 }
 
-var _ = Describe("Block PV upload Test", func() {
+var _ = Describe("Block PV upload Test", Serial, func() {
 	var (
 		pvc *v1.PersistentVolumeClaim
 		err error
@@ -958,6 +1016,10 @@ var _ = Describe("Block PV upload Test", func() {
 	f := framework.NewFramework(namespacePrefix)
 
 	BeforeEach(func() {
+		if !f.IsBlockVolumeStorageClassAvailable() {
+			Skip("Storage Class for block volume is not available")
+		}
+
 		if pvc != nil {
 			Eventually(func() bool {
 				// Make sure the pvc doesn't still exist. The after each should have called delete.
@@ -995,10 +1057,6 @@ var _ = Describe("Block PV upload Test", func() {
 	})
 
 	DescribeTable("should", func(validToken bool, expectedStatus int) {
-		if !f.IsBlockVolumeStorageClassAvailable() {
-			Skip("Storage Class for block volume is not available")
-		}
-
 		By("Verify PVC annotation says ready")
 		found, err := utils.WaitPVCPodStatusReady(f.K8sClient, pvc)
 		Expect(err).ToNot(HaveOccurred())
@@ -1035,7 +1093,7 @@ var _ = Describe("Block PV upload Test", func() {
 	)
 })
 
-var _ = Describe("CDIConfig manipulation upload tests", func() {
+var _ = Describe("CDIConfig manipulation upload tests", Serial, func() {
 	f := framework.NewFramework(namespacePrefix)
 	var (
 		origSpec       *cdiv1.CDIConfigSpec
@@ -1271,7 +1329,7 @@ var _ = Describe("CDIConfig manipulation upload tests", func() {
 
 })
 
-var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:component] Upload tests", func() {
+var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:component] Upload tests", Serial, func() {
 	f := framework.NewFramework("upload-func-test")
 
 	var (
@@ -1700,7 +1758,7 @@ var _ = Describe("[rfe_id:138][crit:high][vendor:cnv-qe@redhat.com][level:compon
 	})
 })
 
-var _ = Describe("Preallocation", func() {
+var _ = Describe("Preallocation", Serial, func() {
 	f := framework.NewFramework(namespacePrefix)
 	dvName := "upload-dv"
 	md5PrefixSize := int64(100000)
